"""
<name>GenCloud</name>
<description>Access expression data from the GenCloud platform.</description>
<icon>icons/GenCloud.svg</icon>
<priority>36</priority>
"""

from __future__ import absolute_import

import Orange
import genapi

import sys, os
from collections import defaultdict
import math
from datetime import date

from Orange.orng import orngEnviron
from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *

import orangecontrib.bio.widgets.OWPIPAx as OWPIPAx

import requests
from orangecontrib.bio import obiDicty

NAME = "GenCloud"
DESCRIPTION = "Access expression data from the GenCloud platform."
ICON = "icons/GenCloud.svg"
PRIORITY = 36

INPUTS = []
OUTPUTS = [("Example table", Orange.data.Table)]

median = obiDicty.median
transformValues = obiDicty.transformValues
averageAttributes = obiDicty.averageAttributes
example_tables = obiDicty.example_tables
CallBack = obiDicty.CallBack

def to_text(x):
    if isinstance(x, dict):
        return str(x["value"])
    else:
        return x

class GenCountConnectionException(Exception):
    pass

class Genesis(object):

    #ignore when joining replicates
    IGNORE_REPLICATE = ["Replicate", "id", "ID", "Name" ]

    #FIXME get this from the server
    LABELS = [("static.name", "Name"),
          ("input.alignment.input.genome.static.name", "Species"),
          ("input.alignment.input.reads.var.sample.strain", "Strain"),
          ("Experiment", "Experiment"),
          ('input.alignment.input.reads.var.sample.genotype', "Genotype"),
          ("input.alignment.input.reads.var.sample.treatment", "Treatment"),
          ("input.alignment.input.reads.var.sample.growth", "Growth"),
          ("input.alignment.input.reads.var.sample.timepoint", "Timepoint"),
          ("input.alignment.input.reads.var.replicates.replicate", "Replicate"),
          ("unique_id", "ID"),
          ("adapter_type", "Adapter"),
          ("experimenter", "Experimenter"),
          ("band", "Band"),
          ("polya", "Polya"),
          ("primer", "Primer"),
          ("shearing", "Shearing")]

    def __init__(self, address, cache=None,
                 username="", password="", connect=True):

        """
        :param str address: The address of the API. 
        :param str username:
        :param str password: Login info; None for public access.
        :param CacheSQLite cache: A cache that stores results locally (an
            :obj:`CacheSQLite`).
        """
        self.address = address
        self.buffer = cache
        self.username = username
        self.password = password
        self._gen = None
        if connect:
            self._gen = genapi.GenCloud(username, password, address)
        self._project = None
        self.projectid = None

    @property
    def gen(self):
        if not self._gen:
            try:
                self._gen = genapi.GenCloud(self.username, self.password, self.address)
            except:
                raise GenCountConnectionException("Connection needed")
        return self._gen

    @property
    def project(self):
        if not self._project or self._project.id != self.projectid:
            self._project = self.gen.projects()[self.projectid]
        return self._project

    def projects(self, reload=False, bufver="0"):
        def a(self):
            return { k:str(p) for k,p in self.gen.projects().items() }
        return self._buffer_fn("projects" + "|||" + self.username, bufver, reload, a, self)

    def result_types(self, reload=False, bufver="0"):
        """Return a list of available result types.
        """
        def a(self):
            objects = self.project.objects(type__startswith='data:expression').values()
            types = set()
            for o in objects:
                an = o.annotation
                for path, a in an.iteritems():
                    if path.startswith('output') and a['type'] == 'basic:file:' \
                        and not path.startswith('output.proc.'):
                            types.add(a["name"])
            return sorted(types)
        return self._buffer_fn(self.projectid + "|||" + self.username + "|||" + "result_types", bufver, reload, a, self)

    def results_list(self, rtype, reload=False, bufver="0"):
        """Return a list of available gene expressions for a specific
        result type. Returns a dictionary, where the keys are ID
        and values are dictionaries of sample annotations.

        :param str rtype: Result type to use (see :obj:`result_types`).
        """
        def a(self):
            objects = self.project.objects(type__startswith='data:expression').values()
            rdict = {}
            for o in objects:
                an = o.annotation
                ok = False
                for path, a in an.iteritems():
                    if path.startswith('output') and a['type'] == 'basic:file:' \
                        and a["name"] == rtype:
                            ok = True  
                if ok:
                    rdict[o.id] = an
                    rdict[o.id]["date_modified"] = o.date_modified
            return rdict
        return self._buffer_fn(self.projectid + "|||" + self.username + "|||" + "results_list"  + "|||" + str(rtype), bufver, reload, a, self)

    def _from_buffer(self, addr):
        return self.buffer.get(self.address + "|v1||" + addr)

    def _to_buffer(self, addr, cont, version="0", autocommit=True):
        if self.buffer:
            return self.buffer.add(self.address + "|v1||" + addr, cont, version=version, autocommit=autocommit)

    def _buffer_commit(self):
        if self.buffer:
            self.buffer.commit()

    def _buffer_fn(self, bufkey, bufver, reload, fn, *args, **kwargs):
        """
        If bufkey is already present in buffer, return its contents.
        If not, run function with arguments and save its result
        into the buffer.
        """
        if self._in_buffer(bufkey) == bufver and reload == False:
            #print "IN", bufkey
            res = self._from_buffer(bufkey)
        else:
            #print "NOT IN", bufkey
            res = fn(*args, **kwargs)
            self._to_buffer(bufkey, res, bufver)
        return res

    def _in_buffer(self, addr):
        if self.buffer:
            return self.buffer.contains(self.address + "|v1||" + addr)
        else:
            return False

    def download(self, ids, rtype, reload=False, bufver="0"):
        objdic = self.results_list(rtype)

        downloads = [] #what to download
        for id in ids:
            o = objdic[id]
            field = None
            for path, a in o.iteritems():
                if path.startswith('output') and a['type'] == 'basic:file:' \
                    and not path.startswith('output.proc.'):
                        if a["name"] == rtype:
                            field = a["value"]["file"]
            downloads.append((id, field))

        bufverfn = (lambda x: bufver) if isinstance(bufver, basestring) else bufver

        unbuffered = [] #what is missing
        for id,field in downloads:
            if not self._in_buffer(id + "|||" + rtype) == bufverfn(id) or reload:
                unbuffered.append((id, field))
        unbufferedset = set(unbuffered)

        newgen = [].__iter__()
        if unbuffered:
            newgen = self.gen.download(unbuffered)
        for id,field in downloads:
            if (id, field) in unbufferedset:
                response = newgen.next()
                out = []
                for l in response.text.split('\n')[1:]:
                    if l:
                        gene, val = l.split('\t')
                        out.append((str(gene), str(val)))
                self._to_buffer(id + "|||" + rtype, out, version=bufverfn(id), autocommit=True)
                yield out
            else:
                yield self._from_buffer(id + "|||" + rtype)

    def get_data(self, ids=None, result_type=None,
                 exclude_constant_labels=False, average=median,
                 callback=None, bufver="0", transform=None,
                 allowed_labels=None, reload=False):
        """
        Return data in a :obj:`Orange.data.Table`. Each feature represents 
        a sample and each row is a gene. The feature's ``.attributes`` 
        contain annotations.

        :param list ids: List of ids as returned by :obj:`results_list`
            if `result_type` is None; list of ids as returned by :obj:`mappings` 
            if `result_type` is set.

        :param str result_type: Result template type id as returned by
             :obj:`result_types`.

        :param bool exclude_constant_labels: If a label has the same value
            in whole example table, remove it.

        :param function average: Function that combines multiple reading of
            the same gene on a chip. If None, no averaging is done.
            Function should take a list of floats and return an "averaged"
            float (the default functions returns the median).

        :param function transform: A function that transforms individual values.
            It should take and return a float. Example use: logarithmic 
            transformation. Default: None.

        """

        def optcb():
            if callback:
                callback()

        cbc = CallBack(len(ids), optcb, callbacks=10)

        res_list = self.results_list(result_type, reload=reload)

        #annotations
        read = {}
        for a in ids:
            an = res_list[a]
            read[a] = []
            for key, pretty in self.LABELS:
                if key in an:
                    read[a].append((pretty, to_text(an[key])))
        cbc.end()
    
        download_func = lambda x: self.download(x, result_type, reload=reload,
                                                bufver=bufver)

        cbc = CallBack(len(ids) + 3, optcb,
                       callbacks=99 - 20)
        et = example_tables(ids, spotmap={}, callback=cbc,
                            annots=read,
                            exclude_constant_labels=exclude_constant_labels,
                            chipfn=download_func,
                            allowed_labels=allowed_labels)
        cbc.end()

        cbc = CallBack(2, optcb, callbacks=10)

        #transformation is performed prior to averaging
        if transform != None:
            transformValues(et, fn=transform)  # in place transform
            cbc()

        #if average function is given, use it to join same spotids
        if average != None:
            et = averageAttributes(et, fn=average)
            cbc()

        cbc.end()

        return et


def tfloat(s):
    try:
        return float(s)
    except:
        return None


class MyTreeWidgetItem(QTreeWidgetItem):

    def __init__(self, parent, *args):
        QTreeWidgetItem.__init__(self, parent, *args)
        self.par = parent

    def __contains__(self, text):
        return any(text.upper() in str(self.text(i)).upper() \
                   for i in range(self.columnCount()))

    def __lt__(self, o1):
        col = self.par.sortColumn()
        if col in [8, 9]:  # WARNING: hardcoded column numbers
            return tfloat(self.text(col)) < tfloat(o1.text(col))
        else:
            return QTreeWidgetItem.__lt__(self, o1)


# set buffer file
bufferpath = os.path.join(orngEnviron.directoryNames["bufferDir"], "genesis")

try:
    os.makedirs(bufferpath)
except:
    pass

bufferfile = os.path.join(bufferpath, "database.sq3")

SelectionByKey = OWPIPAx.SelectionByKey
ListItemDelegate = OWPIPAx.ListItemDelegate
SelectionSetsWidget = OWPIPAx.SelectionSetsWidget
SortedListWidget = OWPIPAx.SortedListWidget
               
# Mapping from PIPAx.results_list annotation keys to Header names.
HEADER = [("_cached", "")] + Genesis.LABELS

# Index of unique_id
ID_INDEX = 10

SORTING_MODEL_LIST = \
    ["Strain", "Experiment", "Genotype",
     "Timepoint", "Growth", "Species",
     "ID", "Name", "Replicate"]


class OWGenCloud(OWWidget):
    settingsList = ["server", "excludeconstant", "username", "password",
                    "joinreplicates", "selectionSetsWidget.selections",
                    "columnsSortingWidget.sortingOrder", "currentSelection",
                    "log2", "experimentsHeaderState", "rtypei", "projecti" ]

    def __init__(self, parent=None, signalManager=None, name="Genesis"):
        OWWidget.__init__(self, parent, signalManager, name)
        self.outputs = [("Example table", ExampleTable)]

        self.username = ""
        self.password = ""
        self.log2 = False
        self.rtypei = 0
        self.projecti = 0

        self.selectedExperiments = []
        self.buffer = obiDicty.CacheSQLite(bufferfile)

        self.searchString = ""
        self.excludeconstant = False
        self.joinreplicates = False
        self.currentSelection = None
    
        self.items = []

        self.experimentsHeaderState = \
                dict(((name, False) for _, name in HEADER[:ID_INDEX + 1]))

        self.result_types = []

        self.controlArea.setMaximumWidth(250)
        self.controlArea.setMinimumWidth(250)
    
        """
        OWGUI.button(self.controlArea, self, "Reload",
                     callback=self.Reload)
        """
        OWGUI.button(self.controlArea, self, "Clear cache",
                     callback=self.clear_cache)

        box = OWGUI.widgetBox(self.controlArea, 'Project')
        self.projectCB = OWGUI.comboBox(box, self, "projecti",
                items=[],
                callback=self.ProjectChosen)

        self.projects = []

        b = OWGUI.widgetBox(self.controlArea, "Experiment Sets")
        self.selectionSetsWidget = SelectionSetsWidget(self)
        self.selectionSetsWidget.setSizePolicy(QSizePolicy.Preferred,
                                               QSizePolicy.Maximum)
        b.layout().addWidget(self.selectionSetsWidget)

        OWGUI.separator(self.controlArea)

        b = OWGUI.widgetBox(self.controlArea, "Sort output columns")
        self.columnsSortingWidget = SortedListWidget(self)
        self.columnsSortingWidget.setSizePolicy(QSizePolicy.Preferred,
                                                QSizePolicy.Maximum)
        b.layout().addWidget(self.columnsSortingWidget)
        sorting_model = QStringListModel(SORTING_MODEL_LIST)
        self.columnsSortingWidget.setModel(sorting_model)

        self.columnsSortingWidget.sortingOrder = \
                ["Strain", "Experiment", "Genotype", "Timepoint"]

        OWGUI.separator(self.controlArea)

        box = OWGUI.widgetBox(self.controlArea, 'Expression Type')
        self.expressionTypesCB = OWGUI.comboBox(box, self, "rtypei",
                items=[],
                callback=self.UpdateResultsList)

        OWGUI.checkBox(self.controlArea, self, "excludeconstant",
                       "Exclude labels with constant values"
                       )

        OWGUI.checkBox(self.controlArea, self, "joinreplicates",
                       "Average replicates (use median)"
                       )

        OWGUI.checkBox(self.controlArea, self, "log2",
                       "Logarithmic (base 2) transformation"
                       )

        self.commit_button = OWGUI.button(self.controlArea, self, "&Commit",
                                          callback=self.Commit)
        self.commit_button.setDisabled(True)

        OWGUI.rubber(self.controlArea)

        box = OWGUI.widgetBox(self.controlArea, "Authentication")

        OWGUI.lineEdit(box, self, "username", "Username:",
                       labelWidth=100,
                       orientation='horizontal',
                       callback=self.AuthChanged)

        self.passf = OWGUI.lineEdit(box, self, "password", "Password:",
                                    labelWidth=100,
                                    orientation='horizontal',
                                    callback=self.AuthChanged)

        self.passf.setEchoMode(QLineEdit.Password)

        OWGUI.lineEdit(self.mainArea, self, "searchString", "Search",
                       callbackOnType=True,
                       callback=self.SearchUpdate)

        self.headerLabels = [t[1] for t in HEADER]

        self.experimentsWidget = QTreeWidget()
        self.experimentsWidget.setHeaderLabels(self.headerLabels)
        self.experimentsWidget.setSelectionMode(QTreeWidget.ExtendedSelection)
        self.experimentsWidget.setRootIsDecorated(False)
        self.experimentsWidget.setSortingEnabled(True)

        contextEventFilter = OWGUI.VisibleHeaderSectionContextEventFilter(
                            self.experimentsWidget, self.experimentsWidget
                            )

        self.experimentsWidget.header().installEventFilter(contextEventFilter)
        self.experimentsWidget.setItemDelegateForColumn(0,
                    OWGUI.IndicatorItemDelegate(self, role=Qt.DisplayRole)
                    )

        self.experimentsWidget.setAlternatingRowColors(True)

        self.connect(self.experimentsWidget.selectionModel(),
                 SIGNAL("selectionChanged(QItemSelection, QItemSelection)"),
                 self.onSelectionChanged)

        self.selectionSetsWidget.setSelectionModel(
                            self.experimentsWidget.selectionModel()
                            )

        self.mainArea.layout().addWidget(self.experimentsWidget)

        self.loadSettings()

        self.restoreHeaderState()

        self.connect(self.experimentsWidget.header(),
                     SIGNAL("geometriesChanged()"),
                     self.saveHeaderState)

        self.dbc = None

        self.AuthSet()

        QTimer.singleShot(100, self.ConnectAndUpdate)

        self.resize(800, 600)

    def AuthSet(self):
        if len(self.username):
            self.passf.setDisabled(False)
        else:
            self.passf.setDisabled(True)

    def AuthChanged(self):
        self.AuthSet()
        self.ConnectAndUpdate()

    def ConnectAndUpdate(self):
        self.Connect()
        if self.dbc:
            self.projects = sorted(self.dbc.projects().items(), key=lambda x: x[1])
            self.UpdateProjects()
            self.ProjectChosen()
            self.UpdateExperimentTypes()

    def Connect(self):
        self.error(1)
        self.warning(1)

        username = "anonymous"
        password = "anonymous"
        if self.username:
            username = self.username
            password = self.password

        self.dbc = None
        self.projects = []
        self.result_types = []

        try:
            self.dbc = Genesis(address="http://cloud.genialis.com/",
                                  username=username,
                                  password=password,
                                  cache=self.buffer)
        except requests.exceptions.ConnectionError:
            self.dbc = Genesis(address="http://cloud.genialis.com/", 
                                  username=username,
                                  password=password,
                                  connect=False,
                                  cache=self.buffer)
            self.warning(1, "Could not connect to server, working from cache.")
        except Exception, ex:
            self.error(1, "Wrong username or password.")

        self.UpdateProjects()
        self.UpdateExperimentTypes() #clear lists

    def Reload(self):
        self.UpdateExperiments(reload=True)

    def clear_cache(self):
        self.buffer.clear()
        self.Reload()

    def rtype(self):
        """Return selected result template type """
        if self.result_types:
            return self.result_types[self.rtypei]
        else:
            return None

    def UpdateExperimentTypes(self):
        self.expressionTypesCB.clear()
        items = [desc for desc  in self.result_types]
        self.expressionTypesCB.addItems(items)
        #do not update anything if the list is empty
        if len(self.result_types):
            self.rtypei = max(0, min(self.rtypei, len(self.result_types) - 1))

    def UpdateProjects(self):
        self.projectCB.clear()
        items = [desc for pid,desc in self.projects]
        self.projectCB.addItems(items)
        #do not update anathing if the list if empty
        if len(self.projects) > 0:
            self.projecti = max(0, min(self.projecti, len(self.projects) - 1))

    def UpdateExperiments(self, reload=False):

        self.experimentsWidget.clear()

        if not self.dbc or not self.dbc.projectid: #the connection did not succeed
            return 

        self.items = []

        self.progressBarInit()

        result_types = []
        sucind = False  # success indicator for database index

        try:
            result_types = self.dbc.result_types(reload=reload)
            sucind = True
        except Exception, ex:
            try:
                result_types = self.dbc.result_types()
                self.warning(0, "Can not access database - using cached data.")
                sucind = True
            except Exception, ex:
                self.error(0, "Can not access database.")

        if sucind:
            self.warning(0)
            self.error(0)

        self.result_types = result_types

        self.UpdateExperimentTypes()
        self.UpdateResultsList(reload=reload)

        self.progressBarFinished()
        
        if self.currentSelection:
            self.currentSelection.select(self.experimentsWidget.selectionModel())

        self.handle_commit_button()

    def ProjectChosen(self, reload=False):
        if self.projects:
            self.dbc.projectid = self.projects[self.projecti][0]
            self.UpdateExperiments(reload=reload)

    def UpdateResultsList(self, reload=False):

        results_list = {}
        results_list = self.dbc.results_list(self.rtype(), reload=reload)
        try:
            results_list = self.dbc.results_list(self.rtype(), reload=reload)
        except Exception, ex:
            try:
                results_list = self.dbc.results_list(self.rtype())
            except Exception, ex:
                self.error(0, "Can not access database.")

        self.results_list = results_list

        #softly change the view so that the selection stays the same

        items_shown = {}
        for i,item in enumerate(self.items):
            c = str(item.text(ID_INDEX))
            items_shown[c] = i

        items_to_show = set(id_ for id_ in self.results_list)

        add_items = set(items_to_show) - set(items_shown)
        delete_items = set(items_shown) - set(items_to_show)

        i = 0
        while i < self.experimentsWidget.topLevelItemCount():
            it = self.experimentsWidget.topLevelItem(i)
            if str(it.text(ID_INDEX)) in delete_items:
                self.experimentsWidget.takeTopLevelItem(i)
            else:
                i += 1

        delete_ind = set([ items_shown[i] for i in delete_items ])
        self.items = [ it for i, it in enumerate(self.items) if i not in delete_ind ]

        for r_annot in add_items:
            d = defaultdict(lambda: "?", self.results_list[r_annot])
            row_items = [""] + [to_text(d.get(key, "?")) for key, _ in HEADER[1:]]
            row_items[ID_INDEX] = r_annot

            ci = MyTreeWidgetItem(self.experimentsWidget, row_items)
            self.items.append(ci)

        for i in range(len(self.headerLabels)):
            self.experimentsWidget.resizeColumnToContents(i)

        self.wantbufver = lambda x: self.results_list[x]["date_modified"]

        self.UpdateCached()


    def UpdateCached(self):

        if self.wantbufver and self.dbc:

            for item in self.items:
                id = str(item.text(ID_INDEX))
                version = self.dbc._in_buffer(id + "|||" + self.rtype())
                value = " " if version == self.wantbufver(id) else ""
                item.setData(0, Qt.DisplayRole, QVariant(value))

    def SearchUpdate(self, string=""):
        for item in self.items:
            item.setHidden(not all(s in item \
                                   for s in self.searchString.split())
                           )

    def Commit(self):

        pb = OWGUI.ProgressBar(self, iterations=100)

        table = None

        ids = []
        for item in self.experimentsWidget.selectedItems():
            unique_id = str(item.text(ID_INDEX))
            ids.append(unique_id)

        transfn = None
        if self.log2:
            transfn = lambda x: math.log(x + 1.0, 2)

        reverse_header_dict = dict((name, key) for key, name in HEADER)

        hview = self.experimentsWidget.header()
        shownHeaders = [label for i, label in \
                        list(enumerate(self.headerLabels))[1:] \
                        if not hview.isSectionHidden(i)
                        ]

        #allowed_labels = [reverse_header_dict.get(label, label) \
        #for label in shownHeaders]
        allowed_labels = None

        #if self.joinreplicates and "id" not in allowed_labels:
        #    # need 'id' labels in join_replicates for attribute names
        #    allowed_labels.append("id")

        if len(ids):
            table = self.dbc.get_data(ids=ids, result_type=self.rtype(),
                          callback=pb.advance,
                          exclude_constant_labels=self.excludeconstant,
                          bufver=self.wantbufver,
                          transform=transfn,
                          allowed_labels=allowed_labels)

            if self.joinreplicates:
                table = obiDicty.join_replicates(table,
                    ignorenames=self.dbc.IGNORE_REPLICATE,
                    namefn=None,
                    avg=obiDicty.median
                    )

            # Sort attributes
            sortOrder = self.columnsSortingWidget.sortingOrder

            all_values = defaultdict(set)
            for at in table.domain.attributes:
                atts = at.attributes
                for name in sortOrder:
                    all_values[name].add(atts.get(reverse_header_dict[name], ""))

            isnum = {}
            for at, vals in all_values.items():
                vals = filter(None, vals)
                try:
                    for a in vals:
                        float(a)
                    isnum[at] = True
                except:
                    isnum[at] = False

            def optfloat(x, at):
                if x == "":
                    return ""
                else:
                    return float(x) if isnum[at] else x

            def sorting_key(attr):
                atts = attr.attributes
                return tuple([optfloat(atts.get(reverse_header_dict[name], ""), name) \
                              for name in sortOrder])

            attributes = sorted(table.domain.attributes,
                                key=sorting_key)

            domain = orange.Domain(attributes, table.domain.classVar)
            domain.addmetas(table.domain.getmetas())
            table = orange.ExampleTable(domain, table)

            from Orange.orng.orngDataCaching import data_hints
            data_hints.set_hint(table, "taxid", "352472")
            data_hints.set_hint(table, "genesinrows", False)

            self.send("Example table", table)

            self.UpdateCached()

        pb.finish()

    def onSelectionChanged(self, selected, deselected):
        self.handle_commit_button()

    def handle_commit_button(self):
        self.currentSelection = \
            SelectionByKey(self.experimentsWidget.selectionModel().selection(),
                           key=(ID_INDEX,))
        self.commit_button.setDisabled(not len(self.currentSelection))

    def saveHeaderState(self):
        hview = self.experimentsWidget.header()
        for i, label in enumerate(self.headerLabels):
            self.experimentsHeaderState[label] = hview.isSectionHidden(i)

    def restoreHeaderState(self):
        hview = self.experimentsWidget.header()
        state = self.experimentsHeaderState
        for i, label in enumerate(self.headerLabels):
            hview.setSectionHidden(i, state.get(label, True))
            self.experimentsWidget.resizeColumnToContents(i)

if __name__ == "__main__":
    app = QApplication(sys.argv)
    obiDicty.verbose = True
    w = OWGenCloud()
    w.show()
    app.exec_()
    w.saveSettings()



from __future__ import absolute_import

import os
import urllib2
import re
import shelve
import shutil
import posixpath
import json
from xml.etree.ElementTree import ElementTree
from StringIO import StringIO

from collections import defaultdict
from functools import partial
from contextlib import closing
from contextlib import contextmanager

from Orange.utils import serverfiles

parse_json = json.load


def parse_xml(stream):
    """ Parse an xml stream into an instance of
    `xml.etree.ElementTree.ElementTree`.
    """
    return ElementTree(file=stream)

# All searchable fields of ArrayExpress (see query_experiments docstring
# for a description of the fields)
ARRAYEXPRESS_FIELDS = \
    ["keywords",
     "accession",
     "array",
     "ef",
     "efv",
     "expdesign",
     "exptype",
     "gxa",
     "pmid",
     "sa",
     "species",
     "expandefo",
     "directsub",
     "assaycount",
     "efcount",
     "samplecount",
     "sacount",
     "rawcount",
     "fgemcount",
     "miamescore",
     "date",
     "wholewords",
     ]


def _open_shelve(filename, flag="r"):
    dirname = os.path.dirname(filename)
    if not os.path.isdir(dirname):
        os.makedirs(dirname)
    exists = os.path.exists(filename)
    if flag in ["r", "w"] and not exists:
        # needs to be created first
        # XXX: Race condition
        s = shelve.open(filename, "c")
        s.close()

    return shelve.open(filename, flag)


class ArrayExpressConnection(object):
    """
    Constructs and runs REST query on ArrayExpress.

    :param address: Address of the ArrayExpress API.
    :param timeout: Timeout for the connection.

    """

    DEFAULT_ADDRESS = "http://www.ebi.ac.uk/arrayexpress/{format}/v2/"
    DEFAULT_FORMAT = "json"
    DEFAULT_CACHE = serverfiles.localpath(
        "ArrayExpress", "ArrayExpressCache.shelve")

    # Order of arguments in the query
    _ARGS_ORDER = ["keywords", "species", "array"]

    def __init__(self, address=None, timeout=30, cache=None,
                 username=None, password=None):
        self.address = address if address is not None else self.DEFAULT_ADDRESS
        self.timeout = timeout
        self.cache = cache if cache is not None else self.DEFAULT_CACHE
        self.username = username
        self.password = password

    def format_query(self, **kwargs):
        """Format the query arguments in `kwargs`.

        >>> conn.format_query(gxa=True, efcount=(1, 5))
        'efcount=[1 TO 5]&gxa=true'

        """
        # Formaters:
        def format_default(val):
            if isinstance(val, basestring):
                return val
            else:
                return "+".join(val)

        def format_species(val):
            return '"%s"' % val.lower()

        def format_gxa(val):
            if val:
                return "true"
            else:
                raise ValueError("gxa={0}".format(val))

        def format_expandefo(val):
            if val:
                return "on"
            else:
                raise ValueError("expandefo={0}".format(val))

        def format_true_false(val):
            return "true" if val else "false"

        def format_interval(val):
            if isinstance(val, tuple):
                return "[{0} TO {1}]".format(*val)
            else:
                raise ValueError("Must be an interval argument (min, max)!")

        def format_date(val):
            # TODO check if val contains a datetime.date object
            # assert proper format
            return format_interval(val)

        def format_wholewords(val):
            if val:
                return "on"
            else:
                raise ValueError("wholewords={0}".format(val))

        formaters = {"species": format_species,
                     "gxa": format_gxa,
                     "expandefo": format_expandefo,
                     "directsub": format_true_false,
                     "assaycount": format_interval,
                     "efcount": format_interval,
                     "samplecount": format_interval,
                     "sacount": format_interval,
                     "rawcount": format_interval,
                     "fgemcount": format_interval,
                     "miamescore": format_interval,
                     "date": format_date,
                     "wholewords": format_wholewords,
                     }
        parts = []
        arg_items = kwargs.items()

        arg_items = sorted(
            arg_items,
            key=lambda arg: self._ARGS_ORDER.index(arg[0])
                            if arg[0] in self._ARGS_ORDER else 100
        )

        for key, value in arg_items:
            if key == "format":
                continue  # format is handled in query_url
            if key not in ARRAYEXPRESS_FIELDS:
                raise ValueError("Invalid argument name: '{0}'".format(key))
            if value is not None and value != []:
                fmt = formaters.get(key, format_default)
                value = fmt(value)
                parts.append("{0}={1}".format(key, value))

        return "&".join(parts)

    def query_url(self, what="experiments", **kwargs):
        """Return a formatted query URL for the query arguments.

        >>> conn.query_url(accession="E-MEXP-31")
        'http://www.ebi.ac.uk/arrayexpress/json/v2/experiments?accession=E-MEXP-31'

        """
        query = self.format_query(**kwargs)
        url = posixpath.join(self.address, what)
        url = url.format(format=kwargs.get("format", self.DEFAULT_FORMAT))
        url = url + ("?" + query if query else "")
        url = url.replace(" ", "%20")
        return url

    def query_url_experiments(self, **kwargs):
        """Return query URL of formatted experiments for the query arguments.
        """
        return self.query_url("experiments", **kwargs)

    def query_url_files(self, **kwargs):
        """ Return query URL of formatted experiments for the query arguments.
        """
        return self.query_url("files", **kwargs)

    def query_experiment(self, **kwargs):
        """Return an open stream to the experiments query results. 
           Takes the same arguments as the :obj:`query_experiments` function.
        """
        url = self.query_url_experiments(**kwargs)
        stream = self._cache_urlopen(url, timeout=self.timeout)
        return stream

    def query_files(self, **kwargs):
        """Return an open stream to the files query results.
           Takes the same arguments as the :obj:`query_files` function.
        """
        url = self.query_url_files(**kwargs)
        stream = self._cache_urlopen(url, timeout=self.timeout)
        return stream

    def open_file(self, accession, kind="raw", ext=None):
        """ Return a file handle to experiment data.
        
        :param str accession:
        :param str kind: Experiment data type.
        
        Possible values for the parameter `kind`:
            - raw: return the raw data if available
            - processed: return the processed data if available
            - biosamples: a png or svg design image
            - idf: investigation description
            - adf: array design description
            - mageml: MAGE-ML file

        Example::

            >>> raw_file = conn.open_file("E-TABM-1087", kind="raw")
            >>> processed_file = conn.open_file("E-TABM-1087", kind="processed")

        """
        stream = self.query_files(accession=accession, format="json")
        data = json.load(stream)
        try:
            files = data["files"]["experiment"]["file"]
        except KeyError:
            raise ValueError(accession)

        for file in files:
            filekind = file["kind"]
            fileext = file["extension"]
            if (filekind == kind) and (fileext == ext or ext is None):
                url = file["url"]
                return self._cache_urlopen(str(url), timeout=self.timeout)

        raise ValueError("%s does not have a file of kind: %r" %
                         (accession, kind))

    def _cache_urlopen(self, url, timeout=30):
        if self.cache is not None:
            with self.open_cache("r") as cache:
                if url in cache:
                    return StringIO(cache[url])

            stream = urllib2.urlopen(url, timeout=timeout)
            data = stream.read()
            with self.open_cache("w") as cache:
                cache[url] = data

            return StringIO(data)
        else:
            return urllib2.urlopen(url, timeout=timeout)

    def open_cache(self, flag="r"):
        if isinstance(self.cache, basestring):
            try:
                return closing(_open_shelve(self.cache, flag))
            except Exception:
                return _fake_closing({})
        elif hasattr(self.cache, "close"):
            return closing(self.cache)
        elif self.cache is None:
            return _fake_closing({})
        else:
            return _fake_closing(self.cache)


@contextmanager
def _fake_closing(obj):
    yield obj


def query_experiments(keywords=None, accession=None, array=None, ef=None,
                      efv=None, expdesign=None, exptype=None,
                      gxa=None, pmid=None, sa=None, species=None,
                      expandefo=None, directsub=None, assaycount=None,
                      efcount=None, samplecount=None, rawcount=None,
                      fgemcount=None, miamescore=None, date=None,
                      format="json", wholewords=None, connection=None):
    """ Query Array Express experiments.

    :param keywords: A list of keywords to search (e.g. ``['gliobastoma']``).
    :param accession: Search by experiment accession (e.g. ``'E-MEXP-31'``).
    :param array: Search by array design name or accession (e.g. ``'A-AFFY-33'``).
    :param ef: Experimental factor (names of main variables of experiments).
    :param efv: Experimental factor value (Has EFO expansion).
    :param expdesign: Experiment design type (e.g. ``["dose", "response"]``).
    :param exptype: Experiment type (e.g. 'RNA-Seq', has EFO expansion).
    :param gxa: If True limit the results to the Gene Expression Atlas only.
    :param pmid: Search by PubMed identifier.
    :param sa: Sample attribute values (e.g. ``'fibroblast'``, has EFO expansion).
    :param species: Search by species (e.g. ``'Homo sapiens'``, has EFO expansion)
    :param expandefo: If True expand the search terms with all its child terms
        in the Experimental Factor Ontology (EFO_) (e.g. ``keywords="cancer"``
        will be expanded to include for synonyms and sub types of cancer).
    :param directsub: If True return only experiments submitted directly to
        Array Express; if False return only experiments imported from GEO
        database; if None (default) return both.
    :param assaycount: A two tuple (min, max) for filter on the number of
        assays (e.g. (1, 5) will return only experiments with at least 1
        and no more then 5 assays).
    :param efcount: Filter on the number of experimental factors (e.g. (1, 5)).
    :param sacount: Filter on the number of sample attribute categories.
    :param rawcount: Filter on the number or raw files.
    :param fgemcount: Filter on the number of final gene expression matrix
        (processed data) files.
    :param miamescore: Filter on the MIAME complience score (max 5).
    :param date: Filter by release date.

    >>> query_experiments(species="Homo sapiens", ef="organism_part", efv="liver")
    {u'experiments': ...

    .. _EFO: http://www.ebi.ac.uk/efo/

    """
    if connection is None:
        connection = ArrayExpressConnection()

    stream = connection.query_experiment(
        keywords=keywords, accession=accession,
        array=array, ef=ef, efv=efv, expdesign=expdesign, exptype=exptype,
        gxa=gxa, pmid=pmid, sa=sa, species=species, expandefo=expandefo,
        directsub=directsub, assaycount=assaycount, efcount=efcount,
        samplecount=samplecount, rawcount=rawcount, fgemcount=fgemcount,
        miamescore=miamescore, date=date, format=format,
        wholewords=wholewords
    )

    if format == "json":
        return parse_json(stream)
    else:
        return parse_xml(stream)


def query_files(keywords=None, accession=None, array=None, ef=None,
                efv=None, expdesign=None, exptype=None,
                gxa=None, pmid=None, sa=None, species=None,
                expandefo=None, directsub=None, assaycount=None,
                efcount=None, samplecount=None, rawcount=None,
                fgemcount=None, miamescore=None, date=None,
                format="json", wholewords=None, connection=None):
    """Query Array Express files. See :obj:`query_experiments` for the
    arguments.

    >>> query_files(species="Mus musculus", ef="developmental_stage",
    ...             efv="embryo", format="xml")
    <xml.etree.ElementTree.ElementTree object ...

    """
    if connection is None:
        connection = ArrayExpressConnection()

    stream = connection.query_files(
        keywords=keywords, accession=accession,
        array=array, ef=ef, efv=efv, expdesign=expdesign, exptype=exptype,
        gxa=gxa, pmid=pmid, sa=sa, species=species, expandefo=expandefo,
        directsub=directsub, assaycount=assaycount, efcount=efcount,
        samplecount=samplecount, rawcount=rawcount, fgemcount=fgemcount,
        miamescore=miamescore, date=date, format=format,
        wholewords=wholewords
    )

    if format == "json":
        return parse_json(stream)
    else:
        return parse_xml(stream)


"""
MAGE-TAB convenience functions, classes
=======================================
"""

IDF_SINGLE_TAGS = \
    ["Investigation Title",
     "Date of Experiment",
     "Public Release Date",
     "Experiment Description",
     ]


def parse_idf(file):
    """ Parse an idf.txt (Investigation Description  Format) formated file.
    Return a list of tuples where the first element is the tag (first column)
    and the second element is a list of all tag values.

    """
    if isinstance(file, basestring):
        file = open(file, "rb")
    data = file.read()
    lines = data.splitlines()
    lines = [line.split("\t") for line in lines
             if line and not line.startswith("#")]
    parsed = [(line[0], line[1:]) for line in lines]
    return parsed


def parse_sdrf(file):
    """ Parse an sdfr formated file. Return a tuple with the first element
    a list of header values and the second element is a list of all rows
    (a row is a list of string values).

    """
    if isinstance(file, basestring):
        file = open(file, "rb")
    data = file.read()
    lines = data.splitlines()
    lines = [line.split("\t") for line in lines
             if line.strip() and not line.startswith("#")]
    header = [h for h in lines[0] if h]
    rows = [line[:len(header)] for line in lines[1:]]
    assert(all([len(r) == len(header) for r in rows]))
    return header, rows


def parse_adf(file):
    raise NotImplementedError


def parse_data_matrix(file):
    """Parse the MAGE-TAB processed data matrix. Return a tuple where the
    elements are:
        - a (header REF, header values) tuple (e.g.
          ("Hybridization REF", ["Stage1", "Stage2", ...]) )
        - a list of quantitation type for header values (e.g.
          ["log2 ratio", "log2 ratio", ...])
        - a (row REF, row names list) tuple ("e.g.
          ("Reporter REF", ["Probe 1", "Probe 2", ...]) )
        - a list of list matrix with values (as strings)

    """
    if isinstance(file, basestring):
        file = open(file, "rb")
    data = file.read()
    lines = data.splitlines()
    lines = [line.split("\t") for line in lines if line.strip()]
    header = lines[0]
    header_ref, header = header[0], header[1:]
    line2 = lines[1]
    row_ref, quant_type = line2[0], line2[1:]
    row_names, rows = [], []
    for line in lines[2:]:
        r_name, row = line[0], line[1:]
        row_names.append(r_name)
        rows.append(row)

    return ((header_ref, header),
            quant_type,
            (row_ref, row_names),
            rows)


class InvestigationDesign(dict):
    r"""Investigation design (contains the contents of the .idf).

    >>> idf_file = StringIO(
    ...     'Investigation Title\tfoo investigation\n' +
    ...     'Experimental Design\tfubar\tsnafu\n' +
    ...     'SDRF File\tfoobar.sdrf\n'
    ... )
    >>> idf = InvestigationDesign(idf_file)
    >>> print idf.investigation_title
    foo investigation
    >>> print idf.experimental_design
    ['fubar', 'snafu']
    >>> print idf.sdrf_file
    ['foobar.sdrf']

    """

    def __init__(self, idf_file=None):
        idf = parse_idf(idf_file)
        self._idf = idf
        self.update(dict(idf))
        for tag, values in idf:
            if tag in IDF_SINGLE_TAGS:
                values = values[0] if values else None
            ttag = self.transform_tag(tag)
            setattr(self, ttag, values)

    def transform_tag(self, tag):
        """ Transform the tag into a proper python attribute name by
        replacing all spaces and special characters (e.g '[', ']' into
        underscores).

        """
        toreplace = [" ", "-", "[", "]"]
        for s in toreplace:
            tag = tag.replace(s, "_")
        return tag.lower()

    def __getitem__(self, tag):
        try:
            return self[tag]
        except KeyError:
            pass

        ttag = self.transform_tag(tag)
        if hasattr(self, ttag):
            return getattr(self, ttag)
        else:
            raise KeyError(tag)


class SampleDataRelationship(object):
    """Sample-Data Relationship (contains the contents of the .sdrf file).

    """

    # Nodes an edges
    NODES_EDGES = [
        "Source Name", "Sample Name", "Extract Name",
        "Labeled Extract Name", "Hybridization Name",
        "Assay Name", "Scan Name", "Normalization Name",
        "Array Data File", "Derived Array Data File",
        "Array Data Matrix File", "Derived Array Data Matrix File",
        "Image File", "Protocol REF"]

    # Attributes for nodes and edges
    NODE_EDGE_ATTRIBUTES = {
        "Source Name": [
            "Characteristics", "Provider", "Material Type",
            "Description", "Comment"],
        "Sample Name": [
            "Characteristics", "Material Type", "Description", "Comment"],
        "Extract Name": [
            "Characteristics", "Material Type", "Description", "Comment"],
        "Labeled Extract Name": [
            "Characteristics", "Material Type", "Description",
            "Label", "Comment"],
        "Hybridization Name": [
            "Array Design File", "Array Design REF", "Comment"],
        "Assay Name": ["Technology Type", "Comment"],
        "Scan Name": ["Comment"],
        "Normalization Name": ["Comment"],
        "Array Data File": ["Comment"],
        "Derived Array Data File": ["Comment"],
        "Array Data Matrix File": ["Comment"],
        "Derived Array Data Matrix File": ["Comment"],
        "Image File": ["Comment"],
        "Protocol REF": [
            "Term Source REF", "Parameter", "Performer", "Date", "Comment"]
    }

    # Attributes
    ATTRIBUTE_COLUMNS = {
        "Characteristics []": ["Unit", "Term Source REF"],
        "Provider": ["Comment"],
        "Material Type": ["Term Source REF"],
        "Label": ["Term Source REF"],
        "Array Design File": ["Comment"],
        "Array Design REF": ["Term Source REF", "Comment"],
        "Technology Type": ["Term Source REF"],
        "Factor Value [] ()": ["Unit", "Term Source REF"],
        "Performer": ["Comment"],
        "Date": [],
        "Parameter Value []": ["Unit", "Comment"],
        "Unit []": ["Term Source REF"],
        "Description": [],
        "Term Source REF": ["Term Accession Number"],
        "Term Accession Number": [],
        "Comment []": []
    }

    def __init__(self, sdrf_file=None):
        header, rows = parse_sdrf(sdrf_file)
        self.header = header
        self.rows = rows

    def transform_tag(self, tag):
        """ Transform the tag into a proper Python attribute name by
        replacing all spaces and special characters (e.g '[', ']' into
        underscores).

        """
        toreplace = [" ", "-", "[", "]"]
        for s in toreplace:
            tag = tag.replace(s, "_")
        return tag.lower()

    def _subsection(self, name):
        """ Return the named subsection (name must be from the
        NODES_EDGES list).

        """
        idx = self.NODES_EDGES.index(name)
        start = self.header.index(name)
        end = -1
        for name in self.NODES_EDGES[idx + 1:]:
            if name in self.header[start + 1:]:
                end = self.header.index(name, start + 1)
                break
        return self.header[start:end], [r[start:end] for r in self.rows]

    def _column(self, name):
        """ Return the named column.
        """
        if isinstance(name, basestring):
            index = self.header.index(name)
        else:
            index = name
        return [r[index] for r in self.rows]

    def source(self):
        """ Return the Source subsection.
        """
        return self._subsection("Source Name")

    def source_name(self):
        """ Return the Source Name subsection.
        """
        return self._column("Source Name")

    def sample(self):
        """ Return the Sample subsection.
        """
        return self._subsection("Sample Name")

    def sample_name(self):
        """ Return the Sample Name subsection
        """
        return self._column("Sample Name")

    def extract(self):
        """ Return the Extract subsection.
        """
        return self._subsection("Extract Name")

    def extract_name(self):
        """ Return the Extract Name subsection.
        """
        return self._column("Extract Name")

    def labeled_extract(self):
        """ Return the Labeled Extract subsection.
        """
        return self._subsection("Labeled Extract Name")

    def labeled_extract_name(self):
        """ Return the Labeled Extract Name subsection.
        """
        return self._column("Labeled Extract Name")

    def hybridization(self):
        """ Return the Hybridization subsection.
        """
        return self._subsection("Hybridization Name")

    def hybridization_name(self):
        """ Return the Hybridization Name subsection.
        """
        return self._column("Hybridization Name")

    def assay(self):
        """ Return the Assay subsection.
        """
        return self._subsection("Assay Name")

    def assay_name(self):
        """ Return the Assay Name subsection.
        """
        return self._column("Assay Name")

    def scan(self):
        """ Return the Scan subsection.
        """
        return self._subsection("Scan Name")

    def scan_name(self):
        """ Return the Scan name subsection.
        """
        return self._column("Scan Name")

    def normalization(self):
        """ Return the Normalization subsection.
        """
        return self._subsection("Normalization Name")

    def normalization_name(self):
        """ Return the Normalization Name subsection.
        """
        return self._column("Normalization Name")

    def array_data(self):
        """ Return the Array Data subsection.
        """
        return self._subsection("Array Data File")

    def array_data_file(self):
        """ Return the Array Data File subsection.
        """
        return self._column("Array Data File")

    def derived_array_data(self):
        """ Return the Derived Array Data subsection.
        """
        return self._subsection("Derived Array Data File")

    def derived_array_data_file(self):
        """ Return the Derived Array Data File subsection.
        """
        return self._column("Derived Array Data File")

    def array_data_matrix(self):
        """ Return the Array Data Matrix subsection.
        """
        return self._subsection("Array Data Matrix File")

    def array_data_matrix_file(self):
        """ Return the Array Data Matrix File subsection.
        """
        return self._column("Array Data Matrix File")

    def derived_array_data_matrix(self):
        """ Return the Derived Array Data Matrix subsection.
        """
        return self._subsection("Derived Array Data Matrix File")

    def derived_array_data_matrix_file(self):
        """ Return the Derived Array Data Matrix File subsection.
        """
        return self._column("Derived Array Data Matrix File")

    def image(self):
        """ Return the Image subsection
        """
        return self._subsection("Image File")

    def image_file(self):
        """ Return the Image File subsection.
        """
        return self._column("Image File")


class ArrayDesign(object):
    """ Array design (contains the contents of the .adf file).

    """

    def __init__(self, adf_file=None):
        adf = parse_adf(adf_file)
        self._adf = adf


def _is_float(str):
    try:
        float(str)
        return True
    except ValueError:
        return False


def _is_continuous(items, check_count=100):
    """ Are the strings in items continuous numbers. 
    """
    count = 0
    i = 0
    for i, item in enumerate(items):
        if _is_float(item):
            count += 1
        if i >= check_count:
            break
    return count >= i * 0.5


def processed_matrix_to_orange(matrix_file, sdrf=None):
    """ Load a single processed matrix file into an :obj:`Orange.data.Table`.
    """
    import numpy
    import Orange

    if isinstance(matrix_file, basestring):
        matrix_file = open(matrix_file, "rb")

    header, quant_type, rows, matrix = parse_data_matrix(matrix_file)
    header_ref, header = header
    row_ref, rows = rows

    matrix = numpy.array(matrix, dtype=object)

    features = []
    is_float = numpy.frompyfunc(_is_float, 1, 1)  # an numpy ufunc

    for header_name, quant, column in zip(header, quant_type, matrix.T):
        if _is_continuous(column):
            feature = Orange.feature.Continuous(header_name)
            # relace all non parsable floats with '?'
            column[numpy.where(1 - is_float(column))] = "?"
        else:
            values = set(column)
            feature = Orange.feature.Discrete(
                header_name, values=sorted(values)
            )
        feature.attributes["quantitation type"] = quant
        features.append(feature)

    row_ref_feature = Orange.feature.String(row_ref)
    domain = Orange.data.Domain(features, None)
    domain.addmeta(Orange.feature.Descriptor.new_meta_id(), row_ref_feature)

    table = Orange.data.Table(domain, [list(row) for row in matrix])
    table.setattr("header_ref", header_ref)
    # Add row identifiers
    for instance, row in zip(table, rows):
        instance[row_ref_feature] = row

    if sdrf is not None:
        pattern = re.compile(
            r"((Characteristics)|(Factor Value)|(Parameter Value)) " +
            r"\[(?P<name>.*?)\].*"
        )
        # Row name in sdrf
        row_name = header_ref[:header_ref.find(" REF")] + " Name"
        # feature names as listed in sdrf
        feature_names = sdrf._column(row_name)
        annotations = defaultdict(partial(defaultdict, set))
        for i, header in enumerate(sdrf.header):
            match = pattern.match(header)
            if match:
                name = match.group("name")
                for val, feature_name in zip(sdrf._column(i), feature_names):
                    annotations[feature_name][name].add(val)

        def to_str(values):
            if len(values) > 1:
                return str(list(values))
            else:
                return str(list(values)[0])

        for feature in table.domain.features:
            feature.attributes.update([(key, to_str(value)) for key, value in \
                                       annotations[feature.name].items()])
    return table


def mage_tab_to_orange(idf_filename):
    """Convert an MAGE-TAB annotated experiment into an 
    :obj:`Orange.data.Table` (assumes all the associated MAGE-TAB 
    files are in the same directory.

    """
    dirname = os.path.dirname(idf_filename)
    idf = InvestigationDesign(idf_filename)

    sdrf_filename = os.path.join(dirname, idf.sdrf_file[0])
    sdrf = SampleDataRelationship(sdrf_filename)

    data_matrices = set(sdrf.derived_array_data_matrix_file())
    data_matrices = [name for name in data_matrices if name.strip()]

    tables = []
    for filename in data_matrices:
        matrix_file = os.path.join(dirname, filename)
        table = processed_matrix_to_orange(matrix_file, sdrf)
        tables.append(table)
    table = hstack_tables(tables)

    return table


def hstack_tables(tables):
    """ Stack the tables horizontally.
    """
    import Orange
    max_len = max([len(table) for table in tables])
    stacked_features = []
    stacked_values = [[] for i in range(max_len)]
    stacked_meta_features = []
    stacked_meta_values = [{} for i in range(max_len)]

    for table in tables:
        stacked_features.extend(table.domain.variables)
        stacked_meta_features.extend(table.domain.getmetas().items())

        for i, instance in enumerate(table):
            stacked_values[i].extend(list(instance))
            stacked_meta_values[i].update(instance.getmetas())

        # Fill extra lines with unknowns
        for i in range(len(table), max_len):
            stacked_values[i].extend(["?"] * len(table.domain.variables))

    domain = Orange.data.Domain(stacked_features, tables[-1].domain.class_var)
    domain.addmetas(dict(set(stacked_meta_features)))
    table = Orange.data.Table(domain, stacked_values)

    # Add meta attributes
    for instance, metas in zip(table, stacked_meta_values):
        for m, val in metas.iteritems():
            instance[m] = val

    return table


def _dictify(element):
    """ 'Dictify' and xml.etree.Element.Element instance by taking all
    subnode tags as keys and the corresponding text values as items i.e. turn
    `<node><bla>foo</bla><a>b</b></node>` into a {"bla":"foo", "a":b"}

    """
    if element is None:
        element = []
    dict = {}
    strip = lambda s: s.strip() if s else s
    for node in element:
        dict[node.tag] = strip(getattr(node, "text", None))
    return dict


class ArrayExpressExperiment(object):

    """
    An convenience class representing an Array Express Experiment.

    >>> ae = ArrayExpressExperiment("E-MEXP-2917")
    >>> print ae.name
    Characterization of Data Variation in Gene Expression Profiling of ...
    >>> for file in ae.files:
    ...     print file["name"], file["url"]
    E-MEXP-2917...
    >>> table = ae.fgem_to_table() # Retrieve the experiment data table

    :param str accession:
        The experiment accession id.

    """

    def __init__(self, accession, connection=None):
        #: Experiment accession id
        self.accession = accession
        #: A list of all species subjugated to torture in this experiment
        self.species = []
        #: True if Array Express provides raw data files.
        self.rawdatafiles = False
        #: True if Array Express provides processed data files.
        self.fgemdatafiles = False
        #: A list of all sample attributes.
        self.sampleattributes = []
        #: A list of experimental factors
        self.experimentalfactors = []

        self.connection = connection
        self._etree = tree = query_experiments(
            accession=accession, connection=self.connection, format="xml")
        experiments = tree.findall("experiment")
        # find the exact match (more then one experiment can be
        # listed in the query result)
        experiments = [e for e in experiments
                       if e.find("accession").text.strip() == accession]
        self._experiment = experiment = experiments[0]

        #: A list of all species subjugated to torture in this experiment
        self.species = [e.text for e in experiment.findall("species")]
        bool_values = {"true": True, "false": False}
        self.rawdatafiles = bool_values[experiment.find("rawdatafiles")
                                        .get("available", "false")]
        self.fgemdatafiles = bool_values[experiment.find("processeddatafiles")
                                         .get("available", "false")]

        for sa in experiment.findall("sampleattribute"):
            category = sa.find("category").text.strip()
            values = [val.text for val in sa.findall("value")]
            self.sampleattributes.append((category, values))

        for ef in experiment.findall("experimentalfactor"):
            name = ef.find("name").text.strip()
            values = [val.text.strip() for val in ef.findall("values")]
            self.experimentalfactors.append((name, values))

        self.miamescores = _dictify(experiment.find("miamescores"))

        self.id = experiment.find("id").text
        self.secondaryaccession = getattr(
            experiment.find("secondaryaccession"), "text", None
        )
        self.name = experiment.find("name").text
        self.experimenttype = experiment.find("experimenttype").text.strip()
        self.releasedate = experiment.find("releasedate").text
        self.lastupdatedate = getattr(
            experiment.find("lastupdatedate"), "text", None
        )
        self.samples = int(experiment.find("samples").text)
        self.assays = int(experiment.find("assays").text)

        self.arraydesign = \
            [_dictify(e) for e in experiment.findall("arraydesign")]

        self.bioassaydatagroups = \
            [_dictify(group)
             for group in experiment.findall("bioassaydatagroup")]
        self.bibliography = \
            [_dictify(e) for e in experiment.findall("bibliography")]
        self.provider = [_dictify(e) for e in experiment.findall("provider")]

        self.experimentdesign = []
        for expd in experiment.findall("experimentdesign"):
            self.experimentdesign.append(expd.text)

        self.description = \
            [_dictify(e) for e in experiment.findall("description")]

        tree = query_files(accession=self.accession,
                           format="xml",
                           connection=self.connection)
        experiments = tree.findall("experiment")
        experiments = [e for e in experiments
                       if e.find("accession").text.strip() == accession]
        experiment = experiments[0]
        files = experiment.findall("file")

        #: A list of file descriptions (dict instances) available provided
        #: by Array Express
        self.files = [_dictify(file) for file in files]

    def _download_file(self, url, extract=True):
        """ Download the `file` from the ArrayExpress into a local
        repository directory.

        """
        rest, basename = posixpath.split(url)
        dirname = posixpath.basename(rest)
        repo_dir = serverfiles.localpath("ArrayExpress", dirname)
        try:
            os.makedirs(repo_dir)
        except OSError:
            pass
        stream = urllib2.urlopen(url)
        local_filename = os.path.join(repo_dir, basename)
        shutil.copyfileobj(stream, open(local_filename, "wb"))

        if extract:
            _, extension = os.path.splitext(local_filename)
            if extension == ".zip":
                import zipfile
                zfile = zipfile.ZipFile(local_filename)
                zfile.extractall(repo_dir)
            elif extension == ".gz":
                import gzip
                gzfile = gzip.open(local_filename)
                gzfile.extractall(repo_dir)
            elif extension in [".tgz"]:
                import tarfile
                tfile = tarfile.TarFile(local_filename)
                tfile.extractall(repo_dir)
            elif extension == ".txt":
                pass
            else:
                raise ValueError("Unknown extension ('{0}').".format(basename))

    def _is_local(self, url):
        """ Is the `url` stored in the local repository?
        """
        return os.path.exists(self._local_filepath(url))

    def _local_filepath(self, url):
        """ Return the local file path for url.
        """
        rest, basename = posixpath.split(url)
        dirname = posixpath.basename(rest)
        return serverfiles.localpath(
                    "ArrayExpress", os.path.join(dirname, basename))

    def _open(self, url):
        """ Return an open file like handle to url (ArrayExpress file).
        The file is cached in the local repository for future access.

        """
        if not self._is_local(url):
            self._download_file(url, extract=True)
        file = self._local_filepath(url)
        return open(file, "rb")

    def _search_files(self, kind=None, extension=None):
        """ Search files by `kind` and `extension`.
        """
        res = []
        for file in self.files:
            kind_match = kind == file.get("kind") or kind is None
            extension_match = (extension == file.get("extension") or
                               extension is None)

            if kind_match and extension_match:
                res.append(file)
        return res

    def array_design(self):
        """ Return a list of :obj:`ArrayDesign` instances used in this experiment.
        """
        files = [f for f in self.files if f.get("kind") == "adf" and
                 f.get("extension") == "txt"]

        array_design = []
        for file in files:
            url = file.get("url")
            if not self._is_local(url):
                self._download_file(url)
            array_design.append(ArrayDesign(self._open(url)))
        return array_design

    def investigation_design(self):
        """ Return an :obj:`InvestigationDesign` for this experiment.
        """
        files = [f for f in self.files if f.get("kind") == "idf" and \
                 f.get("extension") == "txt"]
        if not files:
            raise ValueError("The experiment '{0}' does not have an "
                             "investigation design file"
                             .format(self.accession))
        file = files[0]
        return InvestigationDesign(self._open(file.get("url")))

    def sample_data_relationship(self):
        """
        Return an :obj:`SampleDataRelationship` instance describing this experiment.
        """
        files = [f for f in self.files if f.get("kind") == "sdrf" and \
                 f.get("extension") == "txt"]
        if not files:
            raise ValueError("The experiment '{0}' does not have an sample "
                             "and data relationship file"
                             .format(self.accession))
        file = files[0]
        return SampleDataRelationship(self._open(file.get("url")))

    def fgem_to_table(self):
        """ Retrieve the processed matrix from the Array Express FTP
        server and convert it to a :class:`Orange.data.Table`.

        """
        assert(self.fgemdatafiles)
        repo_dir = serverfiles.localpath("ArrayExpress", self.accession)
        # Find the file listing the data matrix files
        # (should be in sdrf but sometimes it is in 2column file only, why?)
        sdrf = self._search_files("sdrf", "txt")
        if sdrf:
            sdrf = SampleDataRelationship(self._open(sdrf[0].get("url")))
            if "Derived Array Data Matrix File" not in sdrf.header:
                twocol = self._search_files("twocolumn", "txt")
                if twocol:
                    sdrf = SampleDataRelationship(
                        self._open(twocol[0].get("url"))
                    )
        matrix_file = self._search_files("fgem")[0]
        self._open(matrix_file.get("url"))

        idf_file = self._search_files("idf", "txt")[0]
        self._open(idf_file.get("url"))  # To download if not cached
        return mage_tab_to_orange(os.path.join(repo_dir, idf_file.get("name")))


def test():
    conn = ArrayExpressConnection()
    import doctest
    foo = type("foo", (object,), {})()
    bar = type("bar", (object,), {})()

    doctest.testmod(optionflags=doctest.ELLIPSIS,
                    extraglobs={"conn": conn, "foo": foo, "bar": bar})


if __name__ == "__main__":
    test()




from collections import defaultdict
from operator import add
import numpy
import math

def data_type(vals):
    try:
        _ = [ int(a) for a in vals ]
        return int
    except ValueError:
        try:
            _ = [ float(a) for a in vals ]
            return float
        except ValueError:
            return lambda x: x

def separate_by(data, separate, ignore=[], consider=None, add_empty=True):
    """
    data - the data - annotations are saved in the at.attributes
    annotatitions: keys of at.attributes  by which to separate
    ignore: ignore values of these annotations
    consider: consider only these annotations
    """
    ignore = set(ignore)

    annotations = [ at.attributes for at in data.domain.attributes ]

    all_values = defaultdict(set)
    for a in annotations:
        for k,v in a.iteritems():
            all_values[k].add(v)

    types = {}
    for k,vals in all_values.iteritems():
        types[k] = data_type(vals)
    
    groups = defaultdict(list)
    for i,a in enumerate(annotations):
        groups[tuple(a[k] for k in separate)].append(i)

    different_in_all = set(k \
        for k,vals in all_values.iteritems() \
        if len(vals) == len(annotations) or len(vals) == 1)

    other_relevant = set(all_values.keys()) - different_in_all - ignore - set(separate)
    if consider != None:
        other_relevant &= set(consider)
    other_relevant = sorted(other_relevant) #TODO how to order them?

    def relevant_vals(annotation):
        if isinstance(annotation, tuple):
            return annotation
        return tuple(types[v](annotation[v]) for v in other_relevant)

    other_relevant_d2 = defaultdict(int) #"multiset" - number
    #of maximum occurances of a relevant value in a group
    for _,g in groups.items():
        d = defaultdict(int)
        for i in g:
            d[relevant_vals(annotations[i])] += 1
        for rv,n in d.items():
            if n > other_relevant_d2[rv]:
                other_relevant_d2[rv] = n
    
    if add_empty: #fill in with "empty" relevant vals
        ngroups = {}
        for g in groups:
            need_to_fill = other_relevant_d2.copy()
            for i in groups[g]:
                need_to_fill[relevant_vals(annotations[i])] -= 1
            add = []
            for rv,num in need_to_fill.items():
                for a in range(num):
                    add.append(rv)
            ngroups[g] = groups[g] + add
        groups = ngroups

    ngroups = {}
    uniquepos = {} #which positions are unique
    for g in groups:
        elements = list(groups[g])

        rv2 = lambda x: relevant_vals(annotations[x] if isinstance(x,int) else x)

        ngroups[g] = map(lambda x: x if isinstance(x,int) else None,
            sorted(elements, key=rv2))

        d = defaultdict(int) #get groups of different relevant values
        for i in elements:
            d[rv2(i)] += 1
        
        uniquepos[g] = map(lambda x: not d[rv2(x)] > 1,
             sorted(elements, key=rv2))
    
    return ngroups, uniquepos

def float_or_none(value):
    return value.value if value.value != "?" else None

def linearize(data, ids):
    """ Returns a list of floats in the data subspace (or None's
    if the values are unknown or not present. """
    l = [ [ None ] * len(data) if id1 == None \
        else [ float_or_none(ex[id1]) for ex in data ] for id1 in ids ]
    l = reduce(add, l)
    return l

def pearson_lists(l1, l2):
    """ Returns pearson correlation between two lists. Ignores elements
    which are None."""
    okvals = [ (a,b) for a,b in zip(l1,l2) if a != None and b != None ]
    return numpy.corrcoef([ [ v[0] for v in okvals], [ v[1] for v in okvals] ])[0,1]

def euclidean_lists(l1, l2):
    """ Returns pearson correlation between two lists. Ignores elements
    which are None."""
    okvals = [ (a,b) for a,b in zip(l1,l2) if a != None and b != None ]
    return math.sqrt( sum((a-b)*(a-b) for a,b in okvals ))

def spearman_lists(l1, l2):
    """ Returns pearson correlation between two lists. Ignores elements
    which are None."""
    import scipy.stats
    okvals = [ (a,b) for a,b in zip(l1,l2) if a != None and b != None ]
    #print okvals, len(okvals)
    return scipy.stats.spearmanr([ v[0] for v in okvals], [ v[1] for v in okvals] )[0]

def dist_spearman(l1, l2):
    return (1.-spearman_lists(l1, l2))/2

def dist_pcorr(l1, l2):
    #normalized to 0..1
    return (1.-pearson_lists(l1, l2))/2

def dist_eucl(l1, l2):
    return euclidean_lists(l1, l2)



"""
<name>Gene Info</name>
<description>Displays gene information from NCBI and other sources.</description>
<priority>2010</priority>
<contact>Ales Erjavec (ales.erjavec(@at@)fri.uni-lj.si)</contact>
<icon>icons/GeneInfo.svg</icon>
"""

from __future__ import absolute_import, with_statement

import sys
from collections import defaultdict
from functools import partial

from PyQt4.QtCore import pyqtSlot as Slot

import Orange

from Orange.utils import serverfiles
from Orange.utils import lru_cache

from Orange.orng.orngDataCaching import data_hints
from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWGUI import LinkStyledItemDelegate, LinkRole

from Orange.OrangeWidgets.OWWidget import *

from Orange.OrangeWidgets.OWConcurrent import \
    ThreadExecutor, Task, methodinvoke


from .. import gene, taxonomy
from .utils import download


NAME = "Gene Info"
DESCRIPTION = "Displays gene information from NCBI and other sources."
ICON = "icons/GeneInfo.svg"
PRIORITY = 2010

INPUTS = [("Examples", Orange.data.Table, "setData")]
OUTPUTS = [("Selected Examples", Orange.data.Table)]

REPLACES = ["_bioinformatics.widgets.OWGeneInfo.OWGeneInfo"]


class TreeModel(QAbstractItemModel):

    def __init__(self, data, header, parent):
        QAbstractItemModel.__init__(self, parent)
        self._data = [[QVariant(s) for s in row] for row in data]
        self._dataDict = {}
        self._header = header
        self._roleData = {Qt.DisplayRole: self._data}
        self._roleData = partial(
            defaultdict,
            partial(defaultdict,
                    partial(defaultdict, QVariant)))(self._roleData)

    def setColumnLinks(self, column, links):
        font = QFont()
        font.setUnderline(True)
        font = QVariant(font)
        for i, link in enumerate(links):
            self._roleData[LinkRole][i][column] = QVariant(link)
            self._roleData[Qt.FontRole][i][column] = font
            self._roleData[Qt.ForegroundRole][i][column] = \
                QVariant(QColor(Qt.blue))

    def setRoleData(self, role, row, col, data):
        self._roleData[role][row][col] = data

    def data(self, index, role=Qt.DisplayRole):
        row, col = index.row(), index.column()
        return self._roleData[role][row][col]

    def index(self, row, col, parent=QModelIndex()):
        return self.createIndex(row, col, 0)

    def parent(self, index):
        return QModelIndex()

    def rowCount(self, index=QModelIndex()):
        if index.isValid():
            return 0
        else:
            return len(self._data)

    def columnCount(self, index=QModelIndex()):
        return len(self._header)

    def headerData(self, section, orientation, role=Qt.DisplayRole):
        if role == Qt.DisplayRole:
            return QVariant(self._header[section])
        return QVariant()


class LinkFmt(object):

    def __init__(self, link_fmt, name):
        self.link_fmt = link_fmt
        self.name = name

    def format(self, *args, **kwargs):
        return Link(self.link_fmt.format(*args, **kwargs), **kwargs)

    def __repr__(self):
        return "<LinkFmt " + repr(self.name) + " >"

    def __str__(self):
        return self.name


class Link(object):

    def __init__(self, link, text=None, **kwargs):
        self.link = link
        self.text = text if text is not None else "link"
        self.__dict__.update(kwargs)


@lru_cache(maxsize=2)
def get_ncbi_info(taxid):
    return gene.NCBIGeneInfo(taxid)


def ncbi_info(taxid, genes, advance=None):
    taxid = gene.NCBIGeneInfo.TAX_MAP.get(taxid, taxid)
    download.ensure_downloaded(
        "NCBI_geneinfo",
        "gene_info.%s.db" % taxid,
        advance
    )
    info = get_ncbi_info(taxid)

    schema_link = LinkFmt(
        "http://www.ncbi.nlm.nih.gov/sites/entrez?Db=gene&Cmd=ShowDetailView&TermToSearch={gene_id}",
        name="NCBI ID")

    schema = [schema_link, "Symbol", "Locus Tag", "Chromosome",
              "Description", "Synonyms", "Nomenclature"]
    ret = []
    for gene_name in genes:
        gi = info.get_info(gene_name)
        if gi:
            ret.append([schema_link.format(gene_id=gi.gene_id, text=gi.gene_id),
                        gi.symbol + " (%s)" % gene_name if gene_name != gi.symbol else gi.symbol,
                        gi.locus_tag or "",
                        gi.chromosome or "",
                        gi.description or "",
                        ", ".join(gi.synonyms),
                        gi.symbol_from_nomenclature_authority or ""
                        ])
        else:
            ret.append(None)
    return schema, ret


def dicty_info(taxid, genes, advance=None):
    from .. import dicty
    download.ensure_downloaded(
        dicty.DictyBase.domain,
        dicty.DictyBase.filename,
        advance
    )
    info = dicty.DictyBase()
    name_matcher = gene.GMDicty()
    name_matcher.set_targets(info.info.keys())
    schema_link = LinkFmt(
        "http://dictybase.org/db/cgi-bin/gene_page.pl?dictybaseid={gene_id}",
        name="Dicty Base Id")
    schema = [schema_link, "Name", "Synonyms", "Gene Products"]

    ret = []
    for gene_name in genes:
        gene_name = name_matcher.umatch(gene_name)
        gi = info.info.get(gene_name, None)
        if gi:
            ret.append([schema_link.format(gene_id=gene_name, text=gene_name),
                        gi[0] + " (%s)" % gene_name if gene_name != gi[0] else gi[0],  # Gene Name
                        ", ".join(gi[1]),  # Synonyms
                        gi[2] or "",  # Gene Products
                        ])

        else:
            ret.append(None)

    return schema, ret


INFO_SOURCES = {
    "default": [("NCBI Info", ncbi_info)],
    "352472": [("NCBI Info", ncbi_info),
               ("Dicty Base", dicty_info)]
}


class OWGeneInfo(OWWidget):
    settingsList = ["organismIndex", "geneAttr", "useAttr", "autoCommit",
                    "taxid"]
    contextHandlers = {
        "": DomainContextHandler(
            "", ["organismIndex", "geneAttr", "useAttr", "useAltSource",
                 "taxid"]
        )
    }

    def __init__(self, parent=None, signalManager=None, name="Gene Info"):
        OWWidget.__init__(self, parent, signalManager, name)

        self.inputs = [("Examples", Orange.data.Table, self.setData)]
        self.outputs = [("Selected Examples", Orange.data.Table)]

        self.organismIndex = 0
        self.taxid = None
        self.geneAttr = 0
        self.useAttr = False
        self.autoCommit = False
        self.searchString = ""
        self.selectionChangedFlag = False
        self.useAltSource = 0
        self.loadSettings()

        self.__initialized = False
        self.initfuture = None
        self.itemsfuture = None

        self.infoLabel = OWGUI.widgetLabel(
            OWGUI.widgetBox(self.controlArea, "Info", addSpace=True),
            "Initializing\n"
        )

        self.organisms = None
        self.organismBox = OWGUI.widgetBox(
            self.controlArea, "Organism", addSpace=True)

        self.organismComboBox = OWGUI.comboBox(
            self.organismBox, self, "organismIndex",
            callback=self._onSelectedOrganismChanged,
            debuggingEnabled=0)

        # For now only support one alt source, with a checkbox
        # In the future this can be extended to multiple selections
        self.altSourceCheck = OWGUI.checkBox(self.organismBox, self,
                            "useAltSource", "Show information from dictyBase",
                            callback=self.onAltSourceChange,
#                            debuggingEnabled=0,
                            )
        self.altSourceCheck.hide()

        box = OWGUI.widgetBox(self.controlArea, "Gene names", addSpace=True)
        self.geneAttrComboBox = OWGUI.comboBox(
            box, self, "geneAttr",
            "Gene atttibute", callback=self.updateInfoItems
        )
        OWGUI.checkBox(box, self, "useAttr", "Use attribute names",
                       callback=self.updateInfoItems,
                       disables=[(-1, self.geneAttrComboBox)])

        self.geneAttrComboBox.setDisabled(bool(self.useAttr))

        box = OWGUI.widgetBox(self.controlArea, "Commit", addSpace=True)
        b = OWGUI.button(box, self, "Commit", callback=self.commit)
        c = OWGUI.checkBox(box, self, "autoCommit", "Commit on change")
        OWGUI.setStopper(self, b, c, "selectionChangedFlag",
                         callback=self.commit)

        # A label for dictyExpress link
        self.dictyExpressBox = OWGUI.widgetBox(
            self.controlArea, "Dicty Express")
        self.linkLabel = OWGUI.widgetLabel(self.dictyExpressBox, "")
        self.linkLabel.setOpenExternalLinks(False)
        self.connect(self.linkLabel, SIGNAL("linkActivated(QString)"),
                     self.onDictyExpressLink)
        self.dictyExpressBox.hide()

        OWGUI.rubber(self.controlArea)

        OWGUI.lineEdit(self.mainArea, self, "searchString", "Filter",
                       callbackOnType=True, callback=self.searchUpdate)

        self.treeWidget = QTreeView(self.mainArea)
        self.treeWidget.setRootIsDecorated(False)
        self.treeWidget.setSelectionMode(
            QAbstractItemView.ExtendedSelection)
        self.treeWidget.setItemDelegate(
            LinkStyledItemDelegate(self.treeWidget))
        self.treeWidget.setUniformRowHeights(True)
        self.treeWidget.viewport().setMouseTracking(True)
        self.treeWidget.setSortingEnabled(True)
        self.mainArea.layout().addWidget(self.treeWidget)

        box = OWGUI.widgetBox(self.mainArea, "",
                              orientation="horizontal")
        OWGUI.button(box, self, "Select Filtered",
                     callback=self.selectFiltered)
        OWGUI.button(box, self, "Clear Selection",
                     callback=self.treeWidget.clearSelection)

        self.resize(1000, 700)

        self.geneinfo = []
        self.cells = []
        self.row2geneinfo = {}
        self.data = None

        # : (# input genes, # matches genes)
        self.matchedInfo = 0, 0
        self.selectionUpdateInProgress = False

        self.setBlocking(True)
        self.executor = ThreadExecutor(self)

        self.progressBarInit()

        task = Task(
            function=partial(
                taxonomy.ensure_downloaded,
                callback=methodinvoke(self, "advance", ())
            )
        )

        task.resultReady.connect(self.initialize)
        task.exceptionReady.connect(self._onInitializeError)

        self.initfuture = self.executor.submit(task)

    @Slot()
    def advance(self):
        assert self.thread() is QThread.currentThread()

        self.progressBarSet(self.progressBarValue + 1,
                            processEventsFlags=None)

    def initialize(self):
        if self.__initialized:
            # Already initialized
            return

        self.progressBarFinished()

        self.organisms = sorted(
            set([name.split(".")[-2] for name in
                 serverfiles.listfiles("NCBI_geneinfo")] +
                gene.NCBIGeneInfo.essential_taxids())
        )

        self.organismComboBox.addItems(
            [taxonomy.name(tax_id) for tax_id in self.organisms]
        )
        if self.taxid in self.organisms:
            self.organismIndex = self.organisms.index(self.taxid)

        self.infoLabel.setText("No data on input\n")
        self.__initialized = True
        self.initfuture = None

        self.setBlocking(False)

    def _onInitializeError(self, exc):
        sys.excepthook(type(exc), exc.args, None)
        self.error(0, "Could not download the necessary files.")

    def _onSelectedOrganismChanged(self):
        self.taxid = self.organisms[self.organismIndex]
        if self.data is not None:
            self.updateInfoItems()

    def setData(self, data=None):
        if not self.__initialized:
            self.initfuture.result()
            self.initialize()

        if self.itemsfuture is not None:
            raise Exception("Already processing")

        self.closeContext()
        self.data = data

        if data:
            self.geneAttrComboBox.clear()
            self.attributes = \
                [attr for attr in (data.domain.variables +
                                   data.domain.getmetas().values())
                 if isinstance(attr, (Orange.feature.String,
                                      Orange.feature.Discrete))]

            self.geneAttrComboBox.addItems(
                [attr.name for attr in self.attributes]
            )

            self.taxid = data_hints.get_hint(self.data, "taxid", self.taxid)
            self.useAttr = data_hints.get_hint(
                self.data, "genesinrows", self.useAttr)

            self.openContext("", data)
            self.geneAttr = min(self.geneAttr, len(self.attributes) - 1)

            if self.taxid in self.organisms:
                self.organismIndex = self.organisms.index(self.taxid)

            self.updateInfoItems()
        else:
            self.clear()

    def infoSource(self):
        """ Return the current selected info source getter function from
        INFO_SOURCES
        """
        org = self.organisms[min(self.organismIndex, len(self.organisms) - 1)]
        if org not in INFO_SOURCES:
            org = "default"
        sources = INFO_SOURCES[org]
        name, func = sources[min(self.useAltSource, len(sources) - 1)]
        return name, func

    def inputGenes(self):
        if self.useAttr:
            genes = [attr.name for attr in self.data.domain.attributes]
        elif self.attributes:
            attr = self.attributes[self.geneAttr]
            genes = [str(ex[attr]) for ex in self.data
                     if not ex[attr].isSpecial()]
        else:
            genes = []
        return genes

    def updateInfoItems(self):
        self.warning(0)
        if not self.data:
            return

        genes = self.inputGenes()
        if self.useAttr:
            genes = [attr.name for attr in self.data.domain.attributes]
        elif self.attributes:
            attr = self.attributes[self.geneAttr]
            genes = [str(ex[attr]) for ex in self.data
                     if not ex[attr].isSpecial()]
        else:
            genes = []
        if not genes:
            self.warning(0, "Could not extract genes from input dataset.")

        self.warning(1)
        org = self.organisms[min(self.organismIndex, len(self.organisms) - 1)]
        source_name, info_getter = self.infoSource()

        self.error(0)

        self.updateDictyExpressLink(genes, show=org == "352472")
        self.altSourceCheck.setVisible(org == "352472")

        self.progressBarInit()
        self.setBlocking(True)
        self.setEnabled(False)
        self.infoLabel.setText("Retrieving info records.\n")

        self.genes = genes

        task = Task(
            function=partial(
                info_getter, org, genes,
                advance=methodinvoke(self, "advance", ()))
        )
        self.itemsfuture = self.executor.submit(task)
        task.finished.connect(self._onItemsCompleted)

    def _onItemsCompleted(self):
        self.setBlocking(False)
        self.progressBarFinished()
        self.setEnabled(True)

        try:
            schema, geneinfo = self.itemsfuture.result()
        finally:
            self.itemsfuture = None

        self.geneinfo = geneinfo = list(zip(self.genes, geneinfo))
        self.cells = cells = []
        self.row2geneinfo = {}
        links = []
        for i, (_, gi) in enumerate(geneinfo):
            if gi:
                row = []
                for _, item in zip(schema, gi):
                    if isinstance(item, Link):
                        # TODO: This should be handled by delegates
                        row.append(item.text)
                        links.append(item.link)
                    else:
                        row.append(item)
                cells.append(row)
                self.row2geneinfo[len(cells) - 1] = i

        model = TreeModel(cells, [str(col) for col in schema], None)

        model.setColumnLinks(0, links)
        proxyModel = QSortFilterProxyModel(self)
        proxyModel.setSourceModel(model)
        self.treeWidget.setModel(proxyModel)
        self.connect(self.treeWidget.selectionModel(),
                     SIGNAL("selectionChanged(QItemSelection , QItemSelection )"),
                     self.commitIf)

        for i in range(7):
            self.treeWidget.resizeColumnToContents(i)
            self.treeWidget.setColumnWidth(
                i, min(self.treeWidget.columnWidth(i), 200)
            )

        self.infoLabel.setText("%i genes\n%i matched NCBI's IDs" %
                               (len(self.genes), len(cells)))
        self.matchedInfo = len(self.genes), len(cells)

    def clear(self):
        self.infoLabel.setText("No data on input\n")
        self.treeWidget.setModel(
            TreeModel([], ["NCBI ID", "Symbol", "Locus Tag",
                           "Chromosome", "Description", "Synonyms",
                           "Nomenclature"], self.treeWidget))

        self.geneAttrComboBox.clear()
        self.send("Selected Examples", None)

    def commitIf(self, *args):
        if self.autoCommit and not self.selectionUpdateInProgress:
            self.commit()
        else:
            self.selectionChangedFlag = True

    def commit(self):
        if not self.data:
            return
        model = self.treeWidget.model()
        mapToSource = model.mapToSource
        selectedRows = self.treeWidget.selectedIndexes()
        selectedRows = [mapToSource(index).row() for index in selectedRows]
        model = model.sourceModel()

        selectedGeneids = [self.row2geneinfo[row] for row in selectedRows]
        selectedIds = [self.geneinfo[i][0] for i in selectedGeneids]
        selectedIds = set(selectedIds)
        gene2row = dict((self.geneinfo[self.row2geneinfo[row]][0], row)
                        for row in selectedRows)

        if self.useAttr:
            def is_selected(attr):
                return attr.name in selectedIds
            attrs = [attr for attr in self.data.domain.attributes
                     if is_selected(attr)]
            domain = Orange.data.Domain(attrs, self.data.domain.classVar)
            domain.addmetas(self.data.domain.getmetas())
            newdata = Orange.data.Table(domain, self.data)
            self.send("Selected Examples", newdata)
        elif self.attributes:
            attr = self.attributes[self.geneAttr]
            examples = [ex for ex in self.data if str(ex[attr]) in selectedIds]
            # Add gene info
            domain = Orange.data.Domain(
                self.data.domain, self.data.domain.classVar)
            domain.addmetas(self.data.domain.getmetas())
            n_columns = model.columnCount()

            headers = [str(model.headerData(i, Qt.Horizontal, Qt.DisplayRole)
                           .toString())
                       for i in range(n_columns)]
            new_meta_attrs = [(Orange.feature.Descriptor.new_meta_id(),
                               Orange.feature.String(name))
                              for name in headers]
            domain.addmetas(dict(new_meta_attrs))
            examples = [Orange.data.Instance(domain, ex) for ex in examples]
            for ex in examples:
                for i, (_, meta) in enumerate(new_meta_attrs):
                    index = model.index(gene2row[str(ex[attr])], i)
                    ex[meta] = str(
                        model.data(index, Qt.DisplayRole).toString()
                    )

            if examples:
                newdata = Orange.data.Table(examples)
            else:
                newdata = None
            self.send("Selected Examples", newdata)
        else:
            self.send("Selected Examples", None)

    def rowFiltered(self, row):
        searchStrings = self.searchString.lower().split()
        row = unicode(" ".join(self.cells[row]).lower(), errors="ignore")
        return not all([s in row for s in searchStrings])

    def searchUpdate(self):
        if not self.data:
            return
        searchStrings = self.searchString.lower().split()
        index = self.treeWidget.model().sourceModel().index
        mapFromSource = self.treeWidget.model().mapFromSource
        for i, row in enumerate(self.cells):
            row = unicode(" ".join(row).lower(), errors="ignore")
            self.treeWidget.setRowHidden(
                mapFromSource(index(i, 0)).row(),
                QModelIndex(),
                not all([s in row for s in searchStrings]))

    def selectFiltered(self):
        if not self.data:
            return
        itemSelection = QItemSelection()

        index = self.treeWidget.model().sourceModel().index
        mapFromSource = self.treeWidget.model().mapFromSource
        for i, row in enumerate(self.cells):
            if not self.rowFiltered(i):
                itemSelection.select(mapFromSource(index(i, 0)),
                                     mapFromSource(index(i, 0)))
        self.treeWidget.selectionModel().select(
            itemSelection,
            QItemSelectionModel.Select | QItemSelectionModel.Rows)

    def sendReport(self):
        from Orange.OrangeWidgets import OWReport
        genes, matched = self.matchedInfo

        if self.organisms:
            org = self.organisms[min(self.organismIndex,
                                     len(self.organisms) - 1)]
            org_name = taxonomy.name(org)
        else:
            org = None
            org_name = None
        if self.data is not None:
            self.reportRaw(
                "<p>Input: %i genes of which %i (%.1f%%) matched NCBI synonyms"
                "<br>"
                "Organism: %s"
                "<br>"
                "Filter: %s"
                "</p>" % (genes, matched, 100.0 * matched / genes, org_name,
                          self.searchString)
            )
            self.reportSubsection("Gene list")
            self.reportRaw(reportItemView(self.treeWidget))
        else:
            self.reportRaw("<p>No input</p>")

    def updateDictyExpressLink(self, genes, show=False):
        def fix(ddb):
            if ddb.startswith("DDB"):
                if not ddb.startswith("DDB_G"):
                    ddb = ddb.replace("DDB", "DDB_G")
                return ddb
            return None
        if show:
            genes = [fix(gene) for gene in genes if fix(gene)]
            link1 = '<a href="http://dictyexpress.biolab.si/run/index.php?gene=%s">Microarray profile</a>'
            link2 = '<a href="http://dictyexpress.biolab.si/run/index.php?gene=%s&db=rnaseq">RNA-Seq profile</a>'
            self.linkLabel.setText(link1 + "<br/>" + link2)

            show = any(genes)

        if show:
            self.dictyExpressBox.show()
        else:
            self.dictyExpressBox.hide()

    def onDictyExpressLink(self, link):
        if not self.data:
            return

        selectedIndexes = self.treeWidget.selectedIndexes()
        if not len(selectedIndexes):
            QMessageBox.information(
                self, "No gene ids selected",
                "Please select some genes and try again."
            )
            return
        model = self.treeWidget.model()
        mapToSource = model.mapToSource
        selectedRows = self.treeWidget.selectedIndexes()
        selectedRows = [mapToSource(index).row() for index in selectedRows]
        model = model.sourceModel()

        selectedGeneids = [self.row2geneinfo[row] for row in selectedRows]
        selectedIds = [self.geneinfo[i][0] for i in selectedGeneids]
        selectedIds = set(selectedIds)

        def fix(ddb):
            if ddb.startswith("DDB"):
                if not ddb.startswith("DDB_G"):
                    ddb = ddb.replace("DDB", "DDB_G")
                return ddb
            return None

        genes = [fix(gene) for gene in selectedIds if fix(gene)]
        url = str(link) % " ".join(genes)
        QDesktopServices.openUrl(QUrl(url))

    def onAltSourceChange(self):
        self.updateInfoItems()

    def onDeleteWidget(self):
        OWWidget.onDeleteWidget(self)

        # try to cancel pending tasks
        if self.initfuture:
            self.initfuture.cancel()
        if self.itemsfuture:
            self.itemsfuture.cancel()

        self.executor.shutdown()


def reportItemView(view):
    model = view.model()
    return reportItemModel(view, model)


def reportItemModel(view, model, index=QModelIndex()):
    if not index.isValid() or model.hasChildren(index):
        columnCount, rowCount = model.columnCount(index), model.rowCount(index)
        if not index.isValid():
            text = ('<table>\n<tr>' +
                    ''.join('<th>%s</th>' %
                            model.headerData(i, Qt.Horizontal, Qt.DisplayRole)
                            .toString()
                            for i in range(columnCount)) +
                    '</tr>\n')
        else:
            pass
        text += ''.join('<tr>' +
                        ''.join('<td>' + reportItemModel(view, model, model.index(row, column, index)) +
                                '</td>' for column in range(columnCount)) +
                        '</tr>\n'
                        for row in range(rowCount)
                        if not view.isRowHidden(row, index))
        text += '</table>'
        return text
    else:
        variant = model.data(index, Qt.DisplayRole)
        return str(variant.toString()) if variant.isValid() else ""


if __name__ == "__main__":
    app = QApplication(sys.argv)
    data = Orange.data.Table("brown-selected.tab")
    w = OWGeneInfo()
    w.show()

    w.setData(data)
    app.exec_()
    w.saveSettings()



"""<name>Genotype Distances</name>
<description>Compute distances between expression profiles of different experimental factors.</description>
<icon>icons/GenotypeDistances.svg</icon>
<priority>1050</priority>
<contact>Ales Erjavec (ales.erjavec(@at@).fri.uni-lj.si)</contact>
"""

from __future__ import absolute_import

from collections import defaultdict
from operator import add
import math

import numpy

import Orange
from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWItemModels import PyListModel
from Orange.OrangeWidgets.OWWidget import *

from ..obiExperiments import separate_by, data_type, linearize, dist_pcorr, dist_eucl, dist_spearman

NAME = "Expression Profile Distances"
DESCRIPTION = "Compute distances between expression profiles of different experimental factors."
ICON = "icons/GenotypeDistances.svg"
PRIORITY = 1050

INPUTS = [("Example Table", Orange.data.Table, "set_data")]
OUTPUTS = [("Distances", Orange.core.SymMatrix),
           ("Sorted Example Table", Orange.data.Table)]

REPLACES = ["_bioinformatics.widgets.OWGenotypeDistances.OWGenotypeDistances"]


def clone_attr(attr):
    newattr = attr.clone()
    def get_value_from(ex, w=None):
        if attr in ex.domain:
            v = str(ex[attr])
        else:
            v = "?"
        return newattr(v)
        
    newattr.get_value_from = get_value_from
    newattr.source_variable = attr
    newattr.attributes.update(attr.attributes)
    return newattr
    
def name_gen(a):
    i = 1
    while True:
        yield a + " {0}".format(i)
        i += 1
        
missing_name_gen = name_gen("!!missing")
inactive_name_gen = name_gen("!!inactive")


class MyHeaderView(QHeaderView):
    def __init__(self, *args):
        QHeaderView.__init__(self, *args)
        
    def mouseMoveEvent(self, event):
        event.ignore()
        
    def wheelEvent(self, event):
        event.ignore()
        
        
class SetContextHandler(ContextHandler):
    def match(self, context, imperfect, items):
        items = set(items)
        saved_items = set(getattr(context, "items", []))
        if imperfect:
            return float(len(items.intersection(saved_items)))/(len(items.union(saved_items)) or 1)
        else:
            return items == saved_items
        
    def findOrCreateContext(self, widget, items):        
        index, context, score = self.findMatch(widget, self.findImperfect, items)
        if context:
            if index < 0:
                self.addContext(widget, context)
            else:
                self.moveContextUp(widget, index)
            return context, False
        else:
            context = self.newContext()
            context.items = items
            self.addContext(widget, context)
            return context, True
        
class OWGenotypeDistances(OWWidget):
    contextHandlers = {"": SetContextHandler("")}
    settingsList = ["auto_commit"]
    
    DISTANCE_FUNCTIONS = [("Distance from Pearson correlation", dist_pcorr),
                          ("Euclidean distance", dist_eucl),
                          ("Distance from Spearman correlation", dist_spearman)]
    
    def __init__(self, parent=None, signalManager=None, title="Expression Profile Distances"):
        OWWidget.__init__(self, parent, signalManager, title)
        
        self.inputs = [("Example Table", ExampleTable, self.set_data)]
        self.outputs = [("Distances", Orange.core.SymMatrix), ("Sorted Example Table", ExampleTable)]
        
        self.distance_measure = 0
        self.auto_commit = False
        self.changed_flag = False
        
        self.loadSettings()
        
        ########
        # GUI
        ########
        
        self.info_box = OWGUI.widgetLabel(OWGUI.widgetBox(self.controlArea, "Input",
                                                         addSpace=True),
                                         "No data on input\n")
        
        box = OWGUI.widgetBox(self.controlArea, "Separate By",
                              addSpace=True)
        self.separate_view = QListView()
        self.separate_view.setSelectionMode(QListView.MultiSelection)
        box.layout().addWidget(self.separate_view)
        
        box = OWGUI.widgetBox(self.controlArea, "Sort By",
                              addSpace=True)
        self.relevant_view = QListView()
        self.relevant_view.setSelectionMode (QListView.MultiSelection)
        box.layout().addWidget(self.relevant_view)
        
        self.distance_view = OWGUI.comboBox(self.controlArea, self, "distance_measure",
                                            box="Distance Measure",
                                            items=[d[0] for d in self.DISTANCE_FUNCTIONS])
        
        OWGUI.rubber(self.controlArea)
        
        box = OWGUI.widgetBox(self.controlArea, "Commit")
        cb = OWGUI.checkBox(box, self, "auto_commit", "Commit on any change",
                            tooltip="Compute and send the distances on any change.",
                            callback=self.commit_if)
        
        b = OWGUI.button(box, self, "Commit",
                         tooltip="Compute the distances and send the output signals.",
                         callback=self.commit,
                         default=True)
        
        OWGUI.setStopper(self, b, cb, "changed_flag", callback=self.commit)
        
        self.groups_box = OWGUI.widgetBox(self.mainArea, "Groups")
        self.groups_scroll_area = QScrollArea()
        self.groups_box.layout().addWidget(self.groups_scroll_area)
        
        self.data = None
        self.partitions = []
        self.matrix = None
        self.split_groups = []
        self._disable_updates = False
        
        self.resize(800, 600)
        
    def clear(self):
        self.data = None
        self.partitions = []
        self.split_groups = []
        self.matrix = None
        self.send("Distances", None)
        
    def get_suitable_keys(self, data):
        """ Return suitable attr label keys from the data where the key has at least
        two unique values in the data.
        
        """
        attrs = [attr.attributes.items() for attr in data.domain.attributes]
        attrs  = reduce(list.__add__, attrs, [])
        # in case someone put non string values in attributes dict
        attrs = [(str(key), str(value)) for key, value in attrs]
        attrs = set(attrs)
        values = defaultdict(set)
        for key, value in attrs:
            values[key].add(value)
        keys = [key for key in values if len(values[key]) > 1]
        return keys
        
    def set_data(self, data=None):
        """ Set the input example table.
        """
        self.closeContext()
        self.clear()
        self.data = data
        self.error(0)
        self.warning(0)
        if data and not self.get_suitable_keys(data):
            self.error(0, "Data has no suitable attribute labels.")
            data = None
            
        if data:
            self.info_box.setText("{0} genes\n{1} experiments".format(len(data), len(data.domain)))
            self.update_control()
            self.split_data()
        else:
            self.separate_view.setModel(PyListModel([]))
            self.relevant_view.setModel(PyListModel([]))
            self.groups_scroll_area.setWidget(QWidget())
            self.info_box.setText("No data on input.\n")
            self.commit()
            
    def update_control(self):
        """ Update the control area of the widget. Populate the list
        views with keys from attribute labels.
        
        """
        keys = self.get_suitable_keys(self.data)
         
        model = PyListModel(keys)
        self.separate_view.setModel(model)
        self.connect(self.separate_view.selectionModel(),
                     SIGNAL("selectionChanged(QItemSelection, QItemSelection)"),
                     self.on_separate_key_changed)
        
        model = PyListModel(keys)
        self.relevant_view.setModel(model)
        self.connect(self.relevant_view.selectionModel(),
                     SIGNAL("selectionChanged(QItemSelection, QItemSelection)"),
                     self.on_relevant_key_changed)
        
        self.openContext("", keys)
        
        # Get the selected keys from the open context
        context = self.currentContexts[""]
        separate_keys = getattr(context, "separate_keys", set())
        relevant_keys = getattr(context, "relevant_keys", set())
        
        def select(model, selection_model, selected_items):
            all_items = list(model)
            try:
                indices = [all_items.index(item) for item in selected_items]
            except:
                indices = []
            for ind in indices:
                selection_model.select(model.index(ind), QItemSelectionModel.Select)
                
        self._disable_updates = True
        try:
            select(self.relevant_view.model(),
                   self.relevant_view.selectionModel(),
                   relevant_keys)
            
            select(self.separate_view.model(),
                   self.separate_view.selectionModel(),
                   separate_keys)
        finally:
            self._disable_updates = False
        
    def on_separate_key_changed(self, *args):
        if not self._disable_updates:
            context = self.currentContexts[""]
            context.separate_keys = self.selected_separeate_by_keys()
            self.split_data()
    
    def on_relevant_key_changed(self, *args):
        if not self._disable_updates:
            context = self.currentContexts[""]
            context.relevant_keys = self.selected_relevant_keys()
            self.split_data()
        
    def selected_separeate_by_keys(self):
        """ Return the currently selected separate by keys
        """
        rows = self.separate_view.selectionModel().selectedRows()
        rows = sorted([idx.row() for idx in rows])
        keys = [self.separate_view.model()[row] for row in rows]
        return keys
        
    def selected_relevant_keys(self):
        """ Return the currently selected relevant keys
        """
        rows = self.relevant_view.selectionModel().selectedRows()
        rows = sorted([idx.row() for idx in rows])
        keys = [self.relevant_view.model()[row] for row in rows]
        return keys
    
    def split_data(self):
        """ Split the data and update the Groups widget
        """
        separate_keys = self.selected_separeate_by_keys()
        relevant_keys = self.selected_relevant_keys()
        
        self.warning(0)
        if not separate_keys:
            self.warning(0, "No separate by attribute selected.")

        partitions,uniquepos = separate_by(self.data, separate_keys, consider=relevant_keys)
        partitions = partitions.items()

        all_values = defaultdict(set)
        for a in [ at.attributes for at in self.data.domain.attributes ]:
            for k,v in a.iteritems():
                all_values[k].add(v)

        #sort groups
        pkeys = [ key for key,_ in partitions ]
        types = [ data_type([a[i] for a in pkeys]) for i in range(len(pkeys[0])) ]

        partitions = sorted(partitions, key=lambda x:
                    tuple(types[i](v) for i,v in enumerate(x[0])))

        split_groups = []
        
        # Collect relevant key value pairs for all columns
        relevant_items = None

        for keys, indices in partitions:
            if relevant_items == None:
                relevant_items = [ defaultdict(set) for _ in range(len(indices)) ]
            for i, ind in enumerate(indices):
                if ind is not None:
                    attr = self.data.domain[ind]
                    for key in relevant_keys:
                        relevant_items[i][key].add(attr.attributes[key])

        #those with different values between rows are not relevant
        for d in relevant_items:
            for k,s in d.items():
                if len(s) > 1:
                    del d[k]
                else:
                    d[k] = s.pop()

        def get_attr(attr_index, i):
            if attr_index is None:
                attr = Orange.feature.Continuous(missing_name_gen.next())
                attr.attributes.update(relevant_items[i])
                return attr
            else:
                return self.data.domain[attr_index]
        
        for keys, indices in partitions:
            attrs = [get_attr(attr_index, i) for i, attr_index in enumerate(indices)]
            for attr in attrs:
                attr.attributes.update(zip(separate_keys, keys))
            domain = Orange.data.Domain(attrs, None)
            domain.add_metas(self.data.domain.get_metas().items())
#            newdata = Orange.data.Table(domain)
            split_groups.append((keys, domain))
         
        self.set_groups(separate_keys, split_groups, relevant_keys, relevant_items, all_values, uniquepos)
        
        self.partitions = partitions
        self.split_groups = split_groups
        
        self.commit_if()
#        self.update_distances(separate_keys, partitions, self.data)
        
    def set_groups(self, keys, groups, relevant_keys, relevant_items, all_values, uniquepos):
        """ Set the current data groups and update the Group widget
        """
        layout = QVBoxLayout()
        header_widths = []
        header_views = []
        palette = self.palette()
        all_values = all_values.keys()

        def for_print(rd):
            attrs = []
            for d in rd:
                attr = Orange.feature.Continuous(inactive_name_gen.next())
                attr.attributes.update(d)
                attrs.append(attr)
            return Orange.data.Domain(attrs, None)

        for separatev, domain in [ (None, for_print(relevant_items)) ] + groups:
            label = None
            if separatev != None:
                ann_vals = " <b>|</b> ".join(["<b>{0}</ b> = {1}".format(key,val) \
                     for key, val in zip(keys, separatev)])
                label = QLabel(ann_vals)
            
            model = QStandardItemModel()
            for i, attr in enumerate(domain.attributes):
                item = QStandardItem()
                if separatev != None:
                    up = uniquepos[separatev][i]
                else:
                    up = False if False in [ a[i] for a in uniquepos.values() ] else True
                if str(attr.name).startswith("!!missing "): ## TODO: Change this to not depend on name
                    header_text = ["{0}={1}".format(key, attr.attributes.get(key, "?")) \
                                   for key in all_values if key not in relevant_items[i]]
                    header_text = "\n".join(header_text) if header_text else "Empty"
                    item.setData(QVariant(header_text), Qt.DisplayRole)
                    item.setFlags(Qt.NoItemFlags)
                    item.setData(QVariant(QColor(Qt.red)), Qt.ForegroundRole)
                    item.setData(QVariant(palette.color(QPalette.Disabled, QPalette.Window)), Qt.BackgroundRole)
                    item.setData(QVariant("Missing feature."), Qt.ToolTipRole)
                elif str(attr.name).startswith("!!inactive "):
                    header_text = ["{0}={1}".format(key, attr.attributes.get(key, "?")) \
                                   for key in all_values if key in relevant_items[i]]
                    header_text = "\n".join(header_text) if header_text else "No descriptor"
                    item.setData(QVariant(header_text), Qt.DisplayRole)
                    item.setData(QVariant(palette.color(QPalette.Disabled, QPalette.Window)), Qt.BackgroundRole)
                else:
                    header_text = ["{0}={1}".format(key, attr.attributes.get(key, "?")) \
                                   for key in all_values if key not in relevant_items[i]]
                    header_text = "\n".join(header_text) if header_text else "Empty"
                    item.setData(QVariant(header_text), Qt.DisplayRole)
                    item.setData(QVariant(attr.name), Qt.ToolTipRole)

                if up == False:
                    item.setData(QVariant(QColor(Qt.red)), Qt.ForegroundRole)
                    
                model.setHorizontalHeaderItem(i, item)
            attr_count = len(domain.attributes)
            view = MyHeaderView(Qt.Horizontal)
            view.setResizeMode(QHeaderView.Fixed)
            view.setModel(model)
            hint = view.sizeHint()
            view.setMaximumHeight(hint.height())
            
            widths = [view.sectionSizeHint(i) for i in range(attr_count)]
            header_widths.append(widths)
            header_views.append(view)
            
            if label:
                layout.addWidget(label)
            layout.addWidget(view)
            layout.addSpacing(8)
            
        # Make all header sections the same width
        width_sum = 0
        max_header_count = max([h.count() for h in header_views])
        for i in range(max_header_count):
            max_width = max([w[i] for w in header_widths if i < len(w)] or [0])
            for view in header_views:
                if i < view.count():
                    view.resizeSection(i, max_width)
            width_sum += max_width + 2
                
        for h in header_views:
            h.setMinimumWidth(h.length() + 4)
            
        widget = QWidget()
        widget.setLayout(layout)
        widget.setSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Maximum)
        layout.activate()
        
        max_width = max(h.length() for h in header_views) + 20
        
        left, top, right, bottom  = self.getContentsMargins()
        widget.setMinimumWidth(width_sum)
        widget.setMinimumWidth(max_width + left + right)
        self.groups_scroll_area.setWidget(widget)
        
    def compute_distances(self, separate_keys, partitions, data):
        """ Compute the distances between genotypes.
        """
        if separate_keys and partitions:
            self.progressBarInit()
            matrix = Orange.core.SymMatrix(len(partitions))
            profiles = [linearize(data, indices) for _, indices in partitions]
            dist_func = self.DISTANCE_FUNCTIONS[self.distance_measure][1]
            from Orange.utils import progress_bar_milestones
            count = (len(profiles) * len(profiles) - 1) / 2
            milestones = progress_bar_milestones(count)
            iter_count = 0
            for i in range(len(profiles)):
                for j in range(i + 1, len(profiles)):
                    matrix[i, j] = dist_func(profiles[i], profiles[j])
                    iter_count += 1
                    if iter_count in milestones:
                        self.progressBarSet(100.0 * iter_count / count)
            self.progressBarFinished()
            
            items = [["{0}={1}".format(key, value) for key, value in zip(separate_keys, values)] \
                      for values, _ in partitions]
            items = [" | ".join(item) for item in items]
            matrix.setattr("items", items)
        else:
            matrix = None
            
        self.matrix = matrix
        
    def commit_if(self):
        if self.auto_commit and self.changed_flag:
            self.commit()
        else:
            self.changed_flag = True
            
    def commit(self):
        separate_keys = self.selected_separeate_by_keys()
        self.compute_distances(separate_keys,
                               self.partitions,
                               self.data)
        
        if self.split_groups:
            all_attrs = []
            for group, domain in self.split_groups: 
                attrs = []
                group_name = " | ".join("{0}={1}".format(*item) for item in \
                                        zip(separate_keys, group))
                for attr in domain.attributes:
                    newattr = clone_attr(attr)
                    newattr.attributes["<GENOTYPE GROUP>"] = group_name # Need a better way to pass the groups to downstream widgets.
                    attrs.append(newattr)
                    
                all_attrs.extend(attrs)
                
            #all_attrs = reduce(add, [list(domain.attributes) for _, domain in self.split_groups], [])
            domain = Orange.data.Domain(all_attrs, self.data.domain.class_var)
            domain.add_metas(self.data.domain.get_metas().items())
            
            data = Orange.data.Table(domain, self.data)
        else:
            data = None
        self.send("Sorted Example Table", data)
        self.send("Distances", self.matrix)
        self.changed_flag = False

if __name__ == "__main__":
    import os, sys
    app = QApplication(sys.argv )
    w = OWGenotypeDistances()
#    data = Orange.data.Table(os.path.expanduser("~/Documents/dicty-express-sample.tab"))
#    data = Orange.data.Table(os.path.expanduser("~/Downloads/tmp.tab"))
    data = Orange.data.Table(os.path.expanduser("~/tgr.tab"))
    w.set_data(data)
    w.show()
    app.exec_()
    w.saveSettings()
    
#    data = Orange.data.Table("tmp.tab")
#    partitions = separate_by(data, [ "genotype" ], consider=["tp", "replicate"]).items()
#    print partitions
#    l1 = linearize(data, partitions[0][1])
#    l2 = linearize(data, partitions[1][1])
#    print  dist_eucl(l1, l2)
#    print  dist_pcorr(l1, l2)

    




import Orange.bio.geo

gds = Orange.bio.geo.GDS("GDS2960")
data = gds.getdata(sample_type="disease state", transpose=True)
print "Samples: %d, Genes: %d" % (len(data), len(data.domain.attributes))

learners = [ Orange.classification.logreg.LibLinearLogRegLearner ]
results = Orange.evaluation.testing.cross_validation(learners, data, folds=10)
print "AUC = %.3f" % Orange.evaluation.scoring.AUC(results)[0]



"""
============
KEGG Pathway
============

"""
from __future__ import absolute_import

import os
import urllib2

import xml.parsers
from xml.dom import minidom

from contextlib import closing

from Orange.utils import deprecated_attribute

from . import conf
from . import caching
from . import api


def cached_method(func, cache_name="_cached_method_cache", store=None):
    def wrapper(self, *args, **kwargs):
        sig = (func.__name__,) + args + tuple(sorted(kwargs.items()))
        if not hasattr(self, cache_name):
            setattr(self, cache_name, store() if store is not None else {})
        if sig not in getattr(self, cache_name):
            getattr(self, cache_name)[sig] = func(self, *args, **kwargs)
        return getattr(self, cache_name)[sig]
    return wrapper


class Pathway(object):
    """
    Class representing a KEGG Pathway (parsed from a "kgml" file)

    :param str pathway_id: A KEGG pathway id (e.g. 'path:hsa05130')

    """
    KGML_URL_FORMAT = "http://www.genome.jp/kegg-bin/download?entry={pathway_id}&format=kgml"

    def __init__(self, pathway_id, local_cache=None, connection=None):
        if pathway_id.startswith("path:"):
            _, pathway_id = pathway_id.split(":", 1)

        self.pathway_id = pathway_id
        if local_cache is None:
            local_cache = conf.params["cache.path"]
        self.local_cache = local_cache
        self.connection = connection

    def cache_store(self):
        caching.touch_path(self.local_cache)
        return caching.Sqlite3Store(os.path.join(self.local_cache,
                                                 "pathway_store.sqlite3"))

    def _open_last_modified_store(self):
        caching.touch_dir(self.local_cache)
        return caching.Sqlite3Store(os.path.join(self.local_cache,
                                                 "last_modified.sqlite3"))

    def _get_kgml(self):
        """
        Return an open kgml file for the pathway.
        """
        from datetime import datetime, timedelta
        valid = False
        local_filename = os.path.join(self.local_cache,
                                      self.pathway_id + ".xml")
        if os.path.exists(local_filename):
            mtime = os.stat(local_filename).st_mtime
            mtime = datetime.fromtimestamp(mtime)
            now = datetime.now()
            if conf.params["cache.invalidate"] == "always":
                valid = False
            elif conf.params["cache.invalidate"] == "session":
                valid = (now - mtime) < (now - caching._SESSION_START)
            elif conf.params["cache.invalidate"] == "daily":
                valid = (now - mtime) < timedelta(1)
            elif conf.params["cache.invalidate"] == "weekly":
                valid = (now - mtime) < timedelta(7)
            else:
                valid = False

        if not valid:
            url = self.KGML_URL_FORMAT.format(pathway_id=self.pathway_id)
            s = urllib2.urlopen(url)
            contents = s.read()

            with open(local_filename, "wb") as f:
                f.write(contents)

        return open(local_filename, "rb")

    def _get_image_filename(self):
        """
        Return a filename of a local copy of the pathway image
        """
        # TODO: keep-alive (using httplib if it supports it)
        # better to move all code to use requests package

        url = str(self.image)

        local_filename = os.path.join(self.local_cache,
                                      self.pathway_id + ".png")

        if not os.path.exists(local_filename):
            response = urllib2.urlopen(url)
            modified_since = response.headers.get("last-modified")
            image = response.read()
        else:
            request = urllib2.Request(url)
            with closing(self._open_last_modified_store()) as store:
                modified_since = store.get(url, None)

            request.add_header("If-Modified-Since", modified_since)
            try:
                response = urllib2.urlopen(request)
            except urllib2.HTTPError, ex:
                if ex.code == 304:
                    return local_filename
                else:
                    raise
            modified_since = response.headers.get("last-modified")
            image = response.read()

        with open(local_filename, "wb") as f:
            f.write(image)

        with closing(self._open_last_modified_store()) as store:
            store[url] = modified_since

        return local_filename

    def _local_kgml_filename(self):
        """
        Return the local kgml xml filename for the pathway.
        """
        local_filename = os.path.join(self.local_cache,
                                      self.pathway_id + ".xml")
        return local_filename

    class entry(object):
        def __init__(self, dom_element):
            self.__dict__.update(dom_element.attributes.items())
            self.graphics = ()
            self.components = []

            graphics = dom_element.getElementsByTagName("graphics")[0]
            self.graphics = dict(graphics.attributes.items())

            components = dom_element.getElementsByTagName("component")
            self.components = [node.getAttribute("id") for node in components]

    class reaction(object):
        def __init__(self, dom_element):
            self.__dict__.update(dom_element.attributes.items())
            self.substrates = [node.getAttribute("name") for node in
                               dom_element.getElementsByTagName("substrate")]
            self.products = [node.getAttribute("name") for node in
                             dom_element.getElementsByTagName("product")]

    class relation(object):
        def __init__(self, dom_element):
            self.__dict__.update(dom_element.attributes.items())
            self.subtypes = [node.attributes.items() for node in
                             dom_element.getElementsByTagName("subtype")]

    @cached_method
    def pathway_attributes(self):
        return dict(self.pathway_dom().attributes.items())

    @property
    def name(self):
        """
        Pathway name/id (e.g. "path:hsa05130")
        """
        return self.pathway_attributes().get("name")

    @property
    def org(self):
        """
        Pathway organism code (e.g. 'hsa')
        """
        return self.pathway_attributes().get("org")

    @property
    def number(self):
        """
        Pathway number as a string (e.g. '05130')
        """
        return self.pathway_attributes().get("number")

    @property
    def title(self):
        """
        Pathway title string.
        """
        return self.pathway_attributes().get("title")

    @property
    def image(self):
        """
        URL of the pathway image.
        """
        return self.pathway_attributes().get("image")

    @property
    def link(self):
        """
        URL to a pathway on the KEGG web site.
        """
        return self.pathway_attributes().get("link")

    @cached_method
    def pathway_dom(self):
        try:
            return minidom.parse(self._get_kgml()).getElementsByTagName("pathway")[0]
        except xml.parsers.expat.ExpatError:
            # TODO: Should delete the cached xml file.
            return None

    @cached_method
    def entries(self):
        dom = self.pathway_dom()
        if dom:
            return [self.entry(e) for e in dom.getElementsByTagName("entry")]
        else:
            return []

    entrys = deprecated_attribute("entrys", "entries")

    @cached_method
    def reactions(self):
        dom = self.pathway_dom()
        if dom:
            return [self.reaction(e) for e in dom.getElementsByTagName("reaction")]
        else:
            return []

    @cached_method
    def relations(self):
        dom = self.pathway_dom()
        if dom:
            return [self.relation(e) for e in dom.getElementsByTagName("relation")]
        else:
            return []

    def __iter__(self):
        """
        Iterate over all elements in the pathway.
        """
        return iter(self.all_elements())

    def __contains__(self, element):
        """
        Return ``True`` if element in the pathway.
        """
        return element in self.all_elements()

    @classmethod
    def split_pathway_id(cls, id):
        path, id = id.split(":") if ":" in id else ("path", id)
        org, id = id[:-5], id[-5:]
        return path, org, id

    @cached_method
    def all_elements(self):
        """
        Return all elements
        """
        return reduce(list.__add__,
                      [self.genes(), self.compounds(),
                       self.enzmes(), self.reactions()],
                      [])

    def _get_entries_by_type(self, type):
        return sorted(reduce(set.union,
                             [entry.name.split() for entry in self.entries()
                              if entry.type == type],
                             set()))

    @cached_method
    def genes(self):
        """
        Return all genes on the pathway.
        """
        return self._get_entries_by_type("gene")

    @cached_method
    def compounds(self):
        """
        Return all compounds on the pathway.
        """
        return self._get_entries_by_type("compound")

    @cached_method
    def enzymes(self):
        """
        Return all enzymes on the pathway.
        """
        return self._get_entries_by_type("enzyme")

    @cached_method
    def orthologs(self):
        """
        Return all orthologs on the pathway.
        """
        return self._get_entries_by_type("ortholog")

    @cached_method
    def maps(self):
        """
        Return all linked maps on the pathway.
        """
        return self._get_entries_by_type("map")

    @cached_method
    def groups(self):
        """
        Return all groups on the pathway.
        """
        return self._get_entries_by_type("ortholog")

    def get_image(self):
        """
        Return an local filesystem path to an image of the pathway. The image
        will be downloaded if not already cached.
        """
        return self._get_image_filename()

    @classmethod
    def list(cls, organism):
        """
        List all pathways for KEGG organism code `organism`.
        """
        kegg = api.CachedKeggApi()
        return kegg.list_pathways(organism)



## Automatically adapted for numpy.oldnumeric Oct 04, 2007 by 

import math
import numpy.oldnumeric as Numeric, numpy.oldnumeric.mlab as MLab
import numpy.oldnumeric.linear_algebra as LinearAlgebra
import scipy.stats

class ApproxOrthPolyBasis:
    """Approximation of expression profiles with orthogonal polynomials."""

    # for filling missing values in F statistcs (0->p=0, 1e9->p=1)
    _F_fillValue = 1e9

    def __init__(self, points, k=None):
        """sets the points for which the k polynomials of max. degree=k-1 are orthogonal
        default k = number of points
        """
        self.points = points
        if k == None:
            self.k = len(points) #/ 2
        else:
            self.k = k
        self.basis = OrthPolyBasis(self.points, self.k)


    def getAppxCoef(self, arr2d, numCoef=None):
        """returns approx. coeffients given the number of coefficients different from zero"""
        assert len(arr2d.shape) == 2        
        if numCoef == None:
            numCoef = self.k
        else:
            assert 0 <= numCoef <= self.k, "number of coefficients not in range [0," + str(self.k) + "]"
        coef = self.basis.getApproxCoeff(arr2d)
        coef[:,numCoef:] = Numeric.zeros((coef.shape[0], coef.shape[1] - numCoef), Numeric.Float)
        return coef


    def coef_maxCut(self, appxCoef):
        """returns the coefficients different from zero up to the abs. max. coefficient
        where the first coefficient is excluded from finding the max.
        accepts 2d matrix of coefficients where rows represent different curves
        """
        assert len(appxCoef.shape) == 2
        k = Numeric.shape(appxCoef)[1]
        maxInd = Numeric.argmax(Numeric.absolute(appxCoef[:,1:]),1) + 1
        lowDiagOnes = Numeric.fromfunction(lambda i,j: i>=j, (k,k))
        coefSelector = Numeric.take(lowDiagOnes, maxInd, 0)
        return appxCoef*coefSelector


    def getAppxCurve(self, appxCoef, points=None):
        """returns polynomial curve approximation evaluated at given points and given the approx. coefficients
        accepts 2d matrix of coefficients where rows represent different curves
        """
        assert len(appxCoef.shape) == 2
        assert points == None or len(points) > 0, "None or list of points expected"
        return self.basis.evalApproxPoly(appxCoef, points)


    def getAppxCoef3d(self, arr3d, numCoef=None, maxCut=False):
        """returns given number of approx. coefficeints cutted-off at the maximal value (excluding the first one)
        """
        assert len(arr3d.shape) == 3
        if numCoef == None:
            numCoef = self.k
        else:
            assert 0 < numCoef <= self.k, "number of coefficients not in range [1," + str(self.k) + "]"
        coef3d = Numeric.zeros((arr3d.shape[0], self.k, arr3d.shape[2]), Numeric.Float)
        if maxCut:
            for idx2 in range(arr3d.shape[2]):
                coef3d[:,:,idx2] = self.coef_maxCut(self.getAppxCoef(arr3d[:,:,idx2], numCoef=numCoef))
        else:
            for idx2 in range(arr3d.shape[2]):
                coef3d[:,:,idx2] = self.getAppxCoef(arr3d[:,:,idx2], numCoef=numCoef)
        return coef3d


    def getAppxCurve3d(self, appxCoef3d, points=None):
        """returns appx. curves given 3d appx. coefficients in axis 1
        """
        assert len(appxCoef3d.shape) == 3
        assert points == None or len(points) > 0, "None or list of points expected"
        if points == None:
            points = self.points
        curve3d = Numeric.zeros((appxCoef3d.shape[0], len(points), appxCoef3d.shape[2]), Numeric.Float)
        for idx2 in range(appxCoef3d.shape[2]):
            curve3d[:,:,idx2] = self.getAppxCurve(appxCoef3d[:,:,idx2], points)
        return curve3d


    def getAppxCoef2d_pVals(self, arr2d, maxNumCoef):
        """Returns[:,j] the probability that coef. j and all subsequent coef. are equal to 0.
        Reference: Ott, pp.606.
        Use only the first k appx. coef. with pvals below some alpha.
        
        The significance of the coef. estimated by comparing the variance drop of the model2 with the variance of the model1, where:
          model 1: complete model with maxNumCoef coef. different from 0
          model 2: reduced model with coef. in range (0,k) different from 0
        null hypothesis (H0): coef. k,k+1,...,maxNumCoef are equal to 0
            if H0 rejected (pval below some alpha (e.g. 0.05) -> there exist an important coef. among coef. k,k+1,...,maxNumCoef
        repeat the test for k=0,1,...maxNumCoef-1
        """
        assert len(arr2d.shape) == 2, "2d array expected"
        assert 0 < maxNumCoef <= self.k
        coefMax = self.getAppxCoef(arr2d, maxNumCoef)
        curveMax = self.getAppxCurve(coefMax)
        SSE1 = Numeric.add.reduce((arr2d-curveMax)**2,1)
        MSE1 = SSE1 / (arr2d.shape[1]-maxNumCoef)
        #print "SSE1[0], MSE1[0]",SSE1[0], MSE1[0]
        pvals = Numeric.zeros((arr2d.shape[0], maxNumCoef), Numeric.Float)
        for k in range(maxNumCoef):  # test cofInd: [maxNum-1, maxNum-2, ..., minNum]
            #print "Keeping %i coeff" % (k)
            shpk = list(coefMax.shape)
            shpk[1] -= k
            coefk = Numeric.concatenate((coefMax[:,:k], Numeric.zeros((shpk), Numeric.Float)),1)
            curvek = self.getAppxCurve(coefk)
            SSE2 = Numeric.add.reduce((arr2d-curvek)**2,1)
            MSdrop =(SSE2-SSE1) / (maxNumCoef-k)
            F = MSdrop / MSE1
            #2007-10-11: F -> F.filled(???)
            pvals[:,k] = scipy.stats.fprob((maxNumCoef-k), arr2d.shape[1]-maxNumCoef, F.filled(ApproxOrthPolyBasis._F_fillValue))
        return pvals


    def getAppxCoef2d_significant(self, arr2d, maxNumCoef, alpha):
        """Returns[:,j] approx. coeffcients with p-value < alpha; subsequent coef. are equal to 0.
        Reference: Ott, pp.606.
        
        The significance of the coef. estimated by comparing the variance drop of the model2 with the variance of the model1, where:
          model 1: complete model with maxNumCoef coef. different from 0
          model 2: reduced model with coef. in range (0,k) different from 0
        null hypothesis (H0): coef. k,k+1,...,maxNumCoef are equal to 0
            if H0 rejected (pval below some alpha (e.g. 0.05) -> there exist an important coef. among coef. k,k+1,...,maxNumCoef
        repeat the test for k=0,1,...maxNumCoef-1
        """
        assert len(arr2d.shape) == 2, "2d array expected"
        assert 0 < maxNumCoef <= self.k
        coefMax = self.getAppxCoef(arr2d, maxNumCoef)
        curveMax = self.getAppxCurve(coefMax)
        SSE1 = Numeric.add.reduce((arr2d-curveMax)**2,1)
        MSE1 = SSE1 / (arr2d.shape[1]-maxNumCoef)
        #print "SSE1[0], MSE1[0]",SSE1[0], MSE1[0]
        pvals = Numeric.zeros((arr2d.shape[0], maxNumCoef), Numeric.Float)
        for k in range(maxNumCoef):  # test cofInd: [maxNum-1, maxNum-2, ..., minNum]
            #print "Keeping %i coeff" % (k)
            shpk = list(coefMax.shape)
            shpk[1] -= k
            coefk = Numeric.concatenate((coefMax[:,:k], Numeric.zeros((shpk), Numeric.Float)),1)
            curvek = self.getAppxCurve(coefk)
            SSE2 = Numeric.add.reduce((arr2d-curvek)**2,1)
            MSdrop =(SSE2-SSE1) / (maxNumCoef-k)
            F = MSdrop / MSE1
            #2007-10-11: F -> F.filled(???)
            pvals[:,k] = scipy.stats.fprob((maxNumCoef-k), arr2d.shape[1]-maxNumCoef, F.filled(ApproxOrthPolyBasis._F_fillValue))
        pvals = Numeric.where(pvals > alpha, Numeric.resize(Numeric.arange(pvals.shape[1]),pvals.shape), pvals.shape[1])    # MAX where significant, idx where nonsignificant
        firstNonSignIdx = MLab.min(pvals, 1)    # idx of the first non-significant coef.
        coefSign = Numeric.zeros(coefMax.shape, Numeric.Float)
        for idx in range(coefSign.shape[1]):
            coefSign[:,idx] = Numeric.where(idx<firstNonSignIdx, coefMax[:,idx], 0)
        return coefSign
        
    

class OrthPolyBasis:
    """Orthogonal polynomials on a set of distinct points of degree 0 to k-1.
    - best satisfied for x=Numeric.arange(0,4,step) for arbitrary step
    - fragile with increasing k and increasing range of x -> for large i T[i] tends to overflow
    - orthonormal basis not fragile to increasing range of x
    - points mapped to [-1,1]
    TODO: plot() should be smoother !!!
    DONE: go to interval [-1,1], test that k=100 works!!!
    """
    NORM_NONE = 0           # do not normalize the basis polynomials
    NORM_NORM = 1           # generate orthonormal basis: divide base polynomial by sqrt sum squared values at approx. points
    NORM_NORM_T0_1 = 2      # same as NORM_NORM + the first polynomial = 1
    NORM_END1 = 3           # value of all polynomials at the last point equals 1

    def __init__(self, points, k, normalization=NORM_NORM_T0_1, force=False):
        """
        calculate k polynomials of degree 0 to k-1 orthogonal on a set of distinct points
        map points to interval [-1,1]
        INPUT:  points: array of dictinct points where polynomials are orthogonal
                k: number of polynomials of degree 0 to k-1
                force=True creates basis even if orthogonality is not satisfied due to numerical error
        USES:   x: array of points mapped to [-1,1]
                T_: matrix of values of polynomials calculated at x, shape (k,len(x))
                TT_ = T_ * Numeric.transpose(T_)
                TTinv_ = inverse(TT_)
                sc_: scaling factors
                a, b: coefficients for calculating T (2k-4 different from 0, i.e. 6 for k=5)
                n: number of points = len(points)
                normalization = {0|1|2}
        """
        self.k = k  # number of basis polynomials of order 0 to k-1
        self._force = force
        self.points = Numeric.asarray(points, Numeric.Float)
        self.pointsMin = min(points)
        self.pointsMax = max(points)
        # scaling x to [-1,1] results in smaller a and b, T is not affected; overflow is NOT a problem!
        self.xMin = -1
        self.xMax = 1
        self.x = self._map(self.points, self.pointsMin, self.pointsMax, self.xMin, self.xMax)
        # calculate basis polynomials
        self.n = len(points) # the number of approximation points
        t = Numeric.zeros((k,self.n),Numeric.Float)
        a = Numeric.zeros((k,1),Numeric.Float)
        b = Numeric.zeros((k,1),Numeric.Float)
        t[0,:] = Numeric.ones(self.n,Numeric.Float)
        if k > 1: t[1,:] = self.x - sum(self.x)/self.n
        for i in range(1,k-1):
            a[i+1] = Numeric.innerproduct(self.x, t[i,:] * t[i,:]) / Numeric.innerproduct(t[i,:],t[i,:])
            b[i] = Numeric.innerproduct(t[i,:], t[i,:]) / Numeric.innerproduct(t[i-1,:],t[i-1,:])
            t[i+1,:] = (self.x - a[i+1]) * t[i,:] - b[i] * t[i-1,:]
        self.a = a
        self.b = b
        # prepare for approximation
        self._T0 = t
        # orthonormal
        _TT0 = Numeric.matrixmultiply(self._T0, Numeric.transpose(self._T0))
        self.sc1 = Numeric.sqrt(Numeric.reshape(Numeric.diagonal(_TT0),(self.k,1))) # scaling factors = sqrt sum squared self._T0
        self._T1 = self._T0 / self.sc1
        # orthonormal and T[0] == 1
        self.sc2 = Numeric.sqrt(Numeric.reshape(Numeric.diagonal(_TT0),(self.k,1)) / self.n) # scaling factors = sqrt 1/n * sum squared self._T0
        self._T2 = self._T0 / self.sc2
        # T[:,-1] == 1
        self.sc3 = Numeric.take(self._T0, (-1,), 1) # scaling factors = self._T0[:,-1]
        self._T3 = self._T0 / self.sc3
        # set the variables according to the chosen normalization
        self.setNormalization(normalization)


    def getNormalization(self):
        return self._normalization


    def setNormalization(self, normalization):
        if normalization == OrthPolyBasis.NORM_NONE:
            self.T = self._T0
        elif normalization == OrthPolyBasis.NORM_NORM:
            self.T = self._T1
        elif normalization == OrthPolyBasis.NORM_NORM_T0_1:
            self.T = self._T2
        elif normalization == OrthPolyBasis.NORM_END1:
            self.T = self._T3
        else:
            raise "Error: unknown normalization: " + str(normalization)
        self.TT = Numeric.matrixmultiply(self.T, Numeric.transpose(self.T))
        self.TTinv = LinearAlgebra.inverse(self.TT)
        self.TTinvT = Numeric.matrixmultiply(self.TTinv, self.T)
        self.basisCoef = self._getBasisCoef(self.x, self.T)
        self._normalization = normalization
        self._checkOrth(self.T, self.TT, output = self._force)
##        print "self.T", self.T
##        print "self.TT", self.TT
##        print "self.TTinv", self.TTinv
##        print "self.TTinvT", self.TTinvT
        
        

    def _checkOrth(self, T, TT, eps=0.0001, output=False):
        """check if the basis is orthogonal on a set of points x: TT == T*transpose(T) == c*Identity
        INPUT:  T: matrix of values of polynomials calculated at common reference points (x)
                TT = T * transpose(T)
                eps: max numeric error
        """
        TTd0 = (-1.*Numeric.identity(Numeric.shape(TT)[0])+1) * TT  # TTd0 = TT with 0s on the main diagonal
        s = Numeric.sum(Numeric.sum(Numeric.absolute(TTd0)))
        minT = MLab.min(MLab.min(T))
        maxT = MLab.max(MLab.max(T))
        minTTd0 = MLab.min(MLab.min(TTd0))
        maxTTd0 = MLab.max(MLab.max(TTd0))
        if not s < eps:
            out = "NOT ORTHOG, min(T), max(T):\t%f\t%f\n" % (minT, maxT)
            out += "  min(TTd0), max(TTd0), sum-abs-el(TTd0):\t%f\t%f\t%f" % (minTTd0, maxTTd0, s)
            if output:
                print out
                return False
            else:
                raise out
        elif output:
            out = "ORTHOGONAL, min(T), max(T):\t%f\t%f\n" % (minT, maxT)
            out += "  min(TTd0), max(TTd0), sum-abs-el(TTd0):\t%f\t%f\t%f" % (minTTd0, maxTTd0, s)
            print out
        return True


    def zipab(self):
        return zip(Numeric.reshape(self.a,(len(self.a),)).tolist(),Numeric.reshape(self.b, (len(self.b),)).tolist())


    def _map(self, x, a,b, m,M):
        """maps x from the interval [a,b] to the interval [m,M]"""
        x = Numeric.asarray(x, Numeric.Float)
        return (M-m)*(x-a)/(b-a) + m


    #--------APPROXIMATION-------------------------


    def getApproxCoeff(self, curves):
        """curves: 1d or 2d array where each curve in separate line, e.g. curves[curveIdx,timePoints]"""
##        print "curves"
##        print curves
##        print
##        print "Numeric.transpose(self.TTinvT)"
##        print Numeric.transpose(self.TTinvT)
##        print
##        print "Numeric.matrixmultiply(curves, Numeric.transpose(self.TTinvT))"
##        print Numeric.matrixmultiply(curves, Numeric.transpose(self.TTinvT))
        return Numeric.matrixmultiply(curves, Numeric.transpose(self.TTinvT))


    #--------EVALUATION OF POLYNOMIAL-------------------------

    def _getBasisCoef(self, x, T):
        """returns coefficients of basis polynomials given T, i.e. their values at x: (poly0, xi)
        where polynomials in rows, coefficients [a0,a1,...ak] where k=len(x)-1
        similar goes for T
        """
        numPoints = T.shape[1]  # number of points that polynomials are calculated == len(x)
        assert len(x) == numPoints, "len(x)=" + str(x) + ", T.shape[1]=" + str(numPoints) + " do not match"
        numLinSys = T.shape[0]  # number of polynomials
        lin_sys_b = Numeric.transpose(T)
        lin_sys_a = Numeric.ones((numPoints, numPoints), Numeric.Float)
        lin_sys_a[:,1] = x
        for idx in range(2,numPoints):
            lin_sys_a[:,idx] = lin_sys_a[:,idx-1] * x
        return Numeric.transpose(LinearAlgebra.solve_linear_equations(lin_sys_a, lin_sys_b))


    def _evalPolyHorner(self, polyCoef, x):
        """returns (#poly, #x) polynomials evaluated at x where:
            - result: polynomials in rows, values at x in columns: [f(x0)...f(xn)]
            - polyCoef: polynomials in rows, coefficients in columns, sorted by increasing power of x: [c0, c1, ...]
        uses Horner's rule for polynomial evaluation
        """
        x = Numeric.asarray(x, Numeric.Float)
        one_poly = len(polyCoef.shape) == 1
        if one_poly:
            polyCoef = polyCoef[Numeric.NewAxis,:] # [polyIdx, coefIdx]
        val = Numeric.zeros((polyCoef.shape[0],len(x)), Numeric.Float)  # [polyIdx, xIdx]
        for idx in range(polyCoef.shape[1]-1,0,-1):
            val = (val + polyCoef[:,idx:idx+1]) * x
        val += polyCoef[:,0:1]
        if one_poly:
            return val[0,:]
        else:
            return val


    def evalApproxPoly(self, appxCoef, points=None):
        """returns (#curve, #points) an approx. polynomials calculated at points given approx. coeff in rows:
            - appxCoef: curves for approximation in rows, appxCoef in columns [B0, B1, ...]
            - points relative to self.points
        TODO: evaluate on a matrix of appxCoef
        """
        if points == None:
            return Numeric.matrixmultiply(appxCoef, self.T)
        appxCoef = Numeric.asarray(appxCoef, Numeric.Float)
        one_curve = len(appxCoef.shape) == 1
        if one_curve:
            appxCoef = appxCoef[Numeric.NewAxis,:]  # [curveIdx, coefIdx]
        mappedPoints = self._map(points, self.pointsMin, self.pointsMax, self.xMin, self.xMax)
        # eval basis polynomials on mapped points
        basisEval = self._evalPolyHorner(self.basisCoef, mappedPoints) #[basisIdx == coefIdx, pointIdx]
        result = Numeric.matrixmultiply(appxCoef, basisEval)
        if one_curve:
            return result[0,:]
        else:
            return result


class TrigonomerticBasis:
    """Approximation of expression profiles with trigonometric functions orthogonal polynomials."""

    def __init__(self, numPoints, k):
        """numPoints: number of approximation points; k: number of basis functions [2,...,numPoints]"""
        self.numPoints = numPoints
        self.k = k
##        assert k > 1, "Error TrigonomerticBasis: k <= 1"
        assert k <= numPoints, "Error TrigonomerticBasis: k > numPoints"
        # evaluate trigonometric basis functions on the given number of points from [-pi,pi]
        self.x = Numeric.arange(-1*math.pi, math.pi+0.0000001, 2*math.pi/(numPoints-1))
        self.y = Numeric.ones((k, numPoints), Numeric.Float)
        for kk in range(1, k, 2):
##            print "kk, cos %ix" % ((kk+1)/2.)
            self.y[kk] = MLab.cos(self.x*(kk+1)/2) 
        for kk in range(2, k, 2):
##            print "kk, sin %ix" % (kk/2.)
            self.y[kk] = MLab.sin(self.x*kk/2)
        # approx. matrix
        self.Ainv = LinearAlgebra.inverse(Numeric.matrixmultiply(self.y, Numeric.transpose(self.y)))
        self.yyTinvy = Numeric.matrixmultiply(LinearAlgebra.inverse(Numeric.matrixmultiply(self.y, Numeric.transpose(self.y))), self.y)

    def getAppxCoef(self, curves):
        return Numeric.matrixmultiply(curves, Numeric.transpose(self.yyTinvy))

    def getAppxCurve(self, appxCoef):
        return Numeric.matrixmultiply(appxCoef, self.y)

